# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: contrib/envoy/extensions/filters/network/generic_proxy/router/v3/router.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nMcontrib/envoy/extensions/filters/network/generic_proxy/router/v3/router.proto\x12\x38\x65nvoy.extensions.filters.network.generic_proxy.router.v3\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"*\n\x06Router\x12 \n\x18\x62ind_upstream_connection\x18\x01 \x01(\x08\x42\xd9\x01\nFio.envoyproxy.envoy.extensions.filters.network.generic_proxy.router.v3B\x0bRouterProtoP\x01Zpgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/generic_proxy/router/v3;routerv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.generic_proxy.router.v3.router_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\nFio.envoyproxy.envoy.extensions.filters.network.generic_proxy.router.v3B\013RouterProtoP\001Zpgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/generic_proxy/router/v3;routerv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_ROUTER']._serialized_start=203
  _globals['_ROUTER']._serialized_end=245
# @@protoc_insertion_point(module_scope)



# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
from typing import Tuple

import grpc

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x00\x00\x00"

_UNARY_UNARY = "/test/UnaryUnary"
_UNARY_STREAM = "/test/UnaryStream"
_STREAM_UNARY = "/test/StreamUnary"
_STREAM_STREAM = "/test/StreamStream"
STREAM_LENGTH = 5


class _GenericHandler(grpc.GenericRpcHandler):
    def __init__(self):
        self._called = asyncio.get_event_loop().create_future()
        self._routing_table = {
            _UNARY_UNARY: grpc.unary_unary_rpc_method_handler(
                self._unary_unary
            ),
            _UNARY_STREAM: grpc.unary_stream_rpc_method_handler(
                self._unary_stream
            ),
            _STREAM_UNARY: grpc.stream_unary_rpc_method_handler(
                self._stream_unary
            ),
            _STREAM_STREAM: grpc.stream_stream_rpc_method_handler(
                self._stream_stream
            ),
        }

    async def _unary_unary(self, unused_request, unused_context):
        return _RESPONSE

    async def _unary_stream(self, unused_request, unused_context):
        for _ in range(STREAM_LENGTH):
            yield _RESPONSE

    async def _stream_unary(self, request_iterator, unused_context):
        async for _ in request_iterator:
            pass
        return _RESPONSE

    async def _stream_stream(self, request_iterator, servicer_context):
        async for _ in request_iterator:
            yield _RESPONSE

    def service(self, handler_details):
        if not self._called.done():
            self._called.set_result(None)
        return self._routing_table.get(handler_details.method)


async def start_server() -> Tuple[grpc.aio.Server, int]:
    server = grpc.aio.server()
    port = server.add_insecure_port("[::]:0")
    generic_handler = _GenericHandler()
    server.add_generic_rpc_handlers((generic_handler,))
    await server.start()
    return server, port


async def unary_unary_call(port):
    async with grpc.aio.insecure_channel(f"localhost:{port}") as channel:
        multi_callable = channel.unary_unary(_UNARY_UNARY)
        unused_response = await multi_callable(_REQUEST)


async def unary_stream_call(port):
    async with grpc.aio.insecure_channel(f"localhost:{port}") as channel:
        multi_callable = channel.unary_stream(_UNARY_STREAM)
        call = multi_callable(_REQUEST)
        async for _ in call:
            pass


async def stream_unary_call(port):
    async with grpc.aio.insecure_channel(f"localhost:{port}") as channel:
        multi_callable = channel.stream_unary(_STREAM_UNARY)
        call = multi_callable(iter([_REQUEST] * STREAM_LENGTH))
        unused_response = await call


async def stream_stream_call(port):
    async with grpc.aio.insecure_channel(f"localhost:{port}") as channel:
        multi_callable = channel.stream_stream(_STREAM_STREAM)
        call = multi_callable(iter([_REQUEST] * STREAM_LENGTH))
        async for _ in call:
            pass



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of RPC-method-not-found behavior."""

import unittest

from grpc.beta import implementations
from grpc.beta import interfaces
from grpc.framework.interfaces.face import face

from tests.unit.framework.common import test_constants


class NotFoundTest(unittest.TestCase):
    def setUp(self):
        self._server = implementations.server({})
        port = self._server.add_insecure_port("[::]:0")
        channel = implementations.insecure_channel("localhost", port)
        self._generic_stub = implementations.generic_stub(channel)
        self._server.start()

    def tearDown(self):
        self._server.stop(0).wait()
        self._generic_stub = None

    def test_blocking_unary_unary_not_found(self):
        with self.assertRaises(face.LocalError) as exception_assertion_context:
            self._generic_stub.blocking_unary_unary(
                "groop",
                "meffod",
                b"abc",
                test_constants.LONG_TIMEOUT,
                with_call=True,
            )
        self.assertIs(
            exception_assertion_context.exception.code,
            interfaces.StatusCode.UNIMPLEMENTED,
        )

    def test_future_stream_unary_not_found(self):
        rpc_future = self._generic_stub.future_stream_unary(
            "grupe", "mevvod", iter([b"def"]), test_constants.LONG_TIMEOUT
        )
        with self.assertRaises(face.LocalError) as exception_assertion_context:
            rpc_future.result()
        self.assertIs(
            exception_assertion_context.exception.code,
            interfaces.StatusCode.UNIMPLEMENTED,
        )
        self.assertIs(
            rpc_future.exception().code, interfaces.StatusCode.UNIMPLEMENTED
        )


if __name__ == "__main__":
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/endpoint/v3/endpoint_components.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/endpoint/v3/endpoint_components.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import address_pb2 as envoy_dot_config_dot_core_dot_v3_dot_address__pb2
from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.core.v3 import config_source_pb2 as envoy_dot_config_dot_core_dot_v3_dot_config__source__pb2
from envoy.config.core.v3 import health_check_pb2 as envoy_dot_config_dot_core_dot_v3_dot_health__check__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from xds.core.v3 import collection_entry_pb2 as xds_dot_core_dot_v3_dot_collection__entry__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2envoy/config/endpoint/v3/endpoint_components.proto\x12\x18\x65nvoy.config.endpoint.v3\x1a\"envoy/config/core/v3/address.proto\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a(envoy/config/core/v3/config_source.proto\x1a\'envoy/config/core/v3/health_check.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\"xds/core/v3/collection_entry.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xb4\x04\n\x08\x45ndpoint\x12.\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0b\x32\x1d.envoy.config.core.v3.Address\x12Q\n\x13health_check_config\x18\x02 \x01(\x0b\x32\x34.envoy.config.endpoint.v3.Endpoint.HealthCheckConfig\x12\x10\n\x08hostname\x18\x03 \x01(\t\x12R\n\x14\x61\x64\x64itional_addresses\x18\x04 \x03(\x0b\x32\x34.envoy.config.endpoint.v3.Endpoint.AdditionalAddress\x1a\xd2\x01\n\x11HealthCheckConfig\x12\x1d\n\nport_value\x18\x01 \x01(\rB\t\xfa\x42\x06*\x04\x18\xff\xff\x03\x12\x10\n\x08hostname\x18\x02 \x01(\t\x12.\n\x07\x61\x64\x64ress\x18\x03 \x01(\x0b\x32\x1d.envoy.config.core.v3.Address\x12#\n\x1b\x64isable_active_health_check\x18\x04 \x01(\x08:7\x9a\xc5\x88\x1e\x32\n0envoy.api.v2.endpoint.Endpoint.HealthCheckConfig\x1a\x43\n\x11\x41\x64\x64itionalAddress\x12.\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0b\x32\x1d.envoy.config.core.v3.Address:%\x9a\xc5\x88\x1e \n\x1e\x65nvoy.api.v2.endpoint.Endpoint\"\xcc\x02\n\nLbEndpoint\x12\x36\n\x08\x65ndpoint\x18\x01 \x01(\x0b\x32\".envoy.config.endpoint.v3.EndpointH\x00\x12\x17\n\rendpoint_name\x18\x05 \x01(\tH\x00\x12\x39\n\rhealth_status\x18\x02 \x01(\x0e\x32\".envoy.config.core.v3.HealthStatus\x12\x30\n\x08metadata\x18\x03 \x01(\x0b\x32\x1e.envoy.config.core.v3.Metadata\x12\x44\n\x15load_balancing_weight\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x07\xfa\x42\x04*\x02(\x01:\'\x9a\xc5\x88\x1e\"\n envoy.api.v2.endpoint.LbEndpointB\x11\n\x0fhost_identifier\"E\n\x14LbEndpointCollection\x12-\n\x07\x65ntries\x18\x01 \x01(\x0b\x32\x1c.xds.core.v3.CollectionEntry\"r\n\x19LedsClusterLocalityConfig\x12\x37\n\x0bleds_config\x18\x01 \x01(\x0b\x32\".envoy.config.core.v3.ConfigSource\x12\x1c\n\x14leds_collection_name\x18\x02 \x01(\t\"\xa0\x05\n\x13LocalityLbEndpoints\x12\x30\n\x08locality\x18\x01 \x01(\x0b\x32\x1e.envoy.config.core.v3.Locality\x12\x30\n\x08metadata\x18\t \x01(\x0b\x32\x1e.envoy.config.core.v3.Metadata\x12:\n\x0clb_endpoints\x18\x02 \x03(\x0b\x32$.envoy.config.endpoint.v3.LbEndpoint\x12l\n\x17load_balancer_endpoints\x18\x07 \x01(\x0b\x32<.envoy.config.endpoint.v3.LocalityLbEndpoints.LbEndpointListB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0H\x00\x12[\n\x1cleds_cluster_locality_config\x18\x08 \x01(\x0b\x32\x33.envoy.config.endpoint.v3.LedsClusterLocalityConfigH\x00\x12\x44\n\x15load_balancing_weight\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x07\xfa\x42\x04*\x02(\x01\x12\x1a\n\x08priority\x18\x05 \x01(\rB\x08\xfa\x42\x05*\x03\x18\x80\x01\x12/\n\tproximity\x18\x06 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x1aL\n\x0eLbEndpointList\x12:\n\x0clb_endpoints\x18\x01 \x03(\x0b\x32$.envoy.config.endpoint.v3.LbEndpoint:0\x9a\xc5\x88\x1e+\n)envoy.api.v2.endpoint.LocalityLbEndpointsB\x0b\n\tlb_configB\x97\x01\n&io.envoyproxy.envoy.config.endpoint.v3B\x17\x45ndpointComponentsProtoP\x01ZJgithub.com/envoyproxy/go-control-plane/envoy/config/endpoint/v3;endpointv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.endpoint.v3.endpoint_components_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n&io.envoyproxy.envoy.config.endpoint.v3B\027EndpointComponentsProtoP\001ZJgithub.com/envoyproxy/go-control-plane/envoy/config/endpoint/v3;endpointv3\272\200\310\321\006\002\020\002'
  _globals['_ENDPOINT_HEALTHCHECKCONFIG'].fields_by_name['port_value']._loaded_options = None
  _globals['_ENDPOINT_HEALTHCHECKCONFIG'].fields_by_name['port_value']._serialized_options = b'\372B\006*\004\030\377\377\003'
  _globals['_ENDPOINT_HEALTHCHECKCONFIG']._loaded_options = None
  _globals['_ENDPOINT_HEALTHCHECKCONFIG']._serialized_options = b'\232\305\210\0362\n0envoy.api.v2.endpoint.Endpoint.HealthCheckConfig'
  _globals['_ENDPOINT']._loaded_options = None
  _globals['_ENDPOINT']._serialized_options = b'\232\305\210\036 \n\036envoy.api.v2.endpoint.Endpoint'
  _globals['_LBENDPOINT'].fields_by_name['load_balancing_weight']._loaded_options = None
  _globals['_LBENDPOINT'].fields_by_name['load_balancing_weight']._serialized_options = b'\372B\004*\002(\001'
  _globals['_LBENDPOINT']._loaded_options = None
  _globals['_LBENDPOINT']._serialized_options = b'\232\305\210\036\"\n envoy.api.v2.endpoint.LbEndpoint'
  _globals['_LOCALITYLBENDPOINTS'].fields_by_name['load_balancer_endpoints']._loaded_options = None
  _globals['_LOCALITYLBENDPOINTS'].fields_by_name['load_balancer_endpoints']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_LOCALITYLBENDPOINTS'].fields_by_name['load_balancing_weight']._loaded_options = None
  _globals['_LOCALITYLBENDPOINTS'].fields_by_name['load_balancing_weight']._serialized_options = b'\372B\004*\002(\001'
  _globals['_LOCALITYLBENDPOINTS'].fields_by_name['priority']._loaded_options = None
  _globals['_LOCALITYLBENDPOINTS'].fields_by_name['priority']._serialized_options = b'\372B\005*\003\030\200\001'
  _globals['_LOCALITYLBENDPOINTS']._loaded_options = None
  _globals['_LOCALITYLBENDPOINTS']._serialized_options = b'\232\305\210\036+\n)envoy.api.v2.endpoint.LocalityLbEndpoints'
  _globals['_ENDPOINT']._serialized_start=429
  _globals['_ENDPOINT']._serialized_end=993
  _globals['_ENDPOINT_HEALTHCHECKCONFIG']._serialized_start=675
  _globals['_ENDPOINT_HEALTHCHECKCONFIG']._serialized_end=885
  _globals['_ENDPOINT_ADDITIONALADDRESS']._serialized_start=887
  _globals['_ENDPOINT_ADDITIONALADDRESS']._serialized_end=954
  _globals['_LBENDPOINT']._serialized_start=996
  _globals['_LBENDPOINT']._serialized_end=1328
  _globals['_LBENDPOINTCOLLECTION']._serialized_start=1330
  _globals['_LBENDPOINTCOLLECTION']._serialized_end=1399
  _globals['_LEDSCLUSTERLOCALITYCONFIG']._serialized_start=1401
  _globals['_LEDSCLUSTERLOCALITYCONFIG']._serialized_end=1515
  _globals['_LOCALITYLBENDPOINTS']._serialized_start=1518
  _globals['_LOCALITYLBENDPOINTS']._serialized_end=2190
  _globals['_LOCALITYLBENDPOINTS_LBENDPOINTLIST']._serialized_start=2051
  _globals['_LOCALITYLBENDPOINTS_LBENDPOINTLIST']._serialized_end=2127
# @@protoc_insertion_point(module_scope)



# Copyright 2024 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import time
from typing import Sequence

import grpc
from grpc_csm_observability import CsmOpenTelemetryPlugin
from opentelemetry.exporter.prometheus import PrometheusMetricReader
from opentelemetry.sdk.metrics import Histogram
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics import view
from prometheus_client import start_http_server

from examples.python.observability.csm import helloworld_pb2
from examples.python.observability.csm import helloworld_pb2_grpc

logger = logging.getLogger()
console_handler = logging.StreamHandler()
formatter = logging.Formatter(fmt="%(asctime)s: %(levelname)-8s %(message)s")
console_handler.setFormatter(formatter)
logger.addHandler(console_handler)


def _run(target: str, secure_mode: bool, prometheus_endpoint: int):
    csm_plugin = _prepare_csm_observability_plugin(prometheus_endpoint)
    csm_plugin.register_global()
    if secure_mode:
        fallback_creds = grpc.experimental.insecure_channel_credentials()
        channel_creds = grpc.xds_channel_credentials(fallback_creds)
        channel = grpc.secure_channel(target, channel_creds)
    else:
        channel = grpc.insecure_channel(target)
    with channel:
        stub = helloworld_pb2_grpc.GreeterStub(channel)
        # Continuously send RPCs every second.
        while True:
            request = helloworld_pb2.HelloRequest(name="You")
            logger.info("Sending request to server")
            try:
                response = stub.SayHello(request)
                print(f"Greeter client received: {response.message}")
                time.sleep(1)
            except Exception:  # pylint: disable=broad-except
                logger.info(
                    "Request failed, this is normal during initial setup."
                )
    # Deregister is not called in this example, but this is required to clean up.
    csm_plugin.deregister_global()


def _prepare_csm_observability_plugin(
    prometheus_endpoint: int,
) -> CsmOpenTelemetryPlugin:
    # Start Prometheus client
    start_http_server(port=prometheus_endpoint, addr="0.0.0.0")
    reader = PrometheusMetricReader()
    meter_provider = MeterProvider(
        metric_readers=[reader], views=_create_views()
    )
    csm_plugin = CsmOpenTelemetryPlugin(
        meter_provider=meter_provider,
    )
    return csm_plugin


def _create_views() -> Sequence[view.View]:
    """Create a list of views with config for specific metrics."""
    latency_boundaries = [
        0,
        0.00001,
        0.00005,
        0.0001,
        0.0003,
        0.0006,
        0.0008,
        0.001,
        0.002,
        0.003,
        0.004,
        0.005,
        0.006,
        0.008,
        0.01,
        0.013,
        0.016,
        0.02,
        0.025,
        0.03,
        0.04,
        0.05,
        0.065,
        0.08,
        0.1,
        0.13,
        0.16,
        0.2,
        0.25,
        0.3,
        0.4,
        0.5,
        0.65,
        0.8,
        1,
        2,
        5,
        10,
        20,
        50,
        100,
    ]
    size_boundaries = [
        0,
        1024,
        2048,
        4096,
        16384,
        65536,
        262144,
        1048576,
        4194304,
        16777216,
        67108864,
        268435456,
        1073741824,
        4294967296,
    ]
    return [
        view.View(
            instrument_type=Histogram,
            instrument_unit="s",
            aggregation=view.ExplicitBucketHistogramAggregation(
                # Boundaries as defined in gRFC. See:
                # https://github.com/grpc/proposal/blob/master/A66-otel-stats.md
                boundaries=latency_boundaries
            ),
        ),
        view.View(
            instrument_type=Histogram,
            instrument_unit="By",
            aggregation=view.ExplicitBucketHistogramAggregation(
                # Boundaries as defined in gRFC. See:
                # https://github.com/grpc/proposal/blob/master/A66-otel-stats.md
                boundaries=size_boundaries
            ),
        ),
    ]


def bool_arg(arg: str) -> bool:
    if arg.lower() in ("true", "yes", "y"):
        return True
    elif arg.lower() in ("false", "no", "n"):
        return False
    else:
        raise argparse.ArgumentTypeError(f"Could not parse '{arg}' as a bool.")


if __name__ == "__main__":
    logging.basicConfig()
    logger.setLevel(logging.INFO)
    parser = argparse.ArgumentParser(
        description="Run Python CSM Observability Test client."
    )
    parser.add_argument(
        "--target",
        default="localhost:50051",
        help="The address of the server.",
    )
    parser.add_argument(
        "--secure_mode",
        default="False",
        type=bool_arg,
        help="If specified, uses xDS credentials to connect to the server.",
    )
    parser.add_argument(
        "--prometheus_endpoint",
        type=int,
        default=9464,
        help="Port for servers besides test server.",
    )
    args = parser.parse_args()
    _run(args.target, args.secure_mode, args.prometheus_endpoint)



# Copyright 2019 The gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import logging
import unittest

from grpc.experimental import aio

from src.proto.grpc.testing import benchmark_service_pb2_grpc
from tests_aio.benchmark import benchmark_servicer


async def _start_async_server():
    server = aio.server()

    port = server.add_insecure_port("localhost:%s" % 50051)
    servicer = benchmark_servicer.BenchmarkServicer()
    benchmark_service_pb2_grpc.add_BenchmarkServiceServicer_to_server(
        servicer, server
    )

    await server.start()
    logging.info("Benchmark server started at :%d" % port)
    await server.wait_for_termination()


def main():
    loop = asyncio.get_event_loop()
    loop.create_task(_start_async_server())
    loop.run_forever()


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cel/expr/conformance/conformance_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cel/expr/conformance/conformance_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cel.expr import checked_pb2 as cel_dot_expr_dot_checked__pb2
from cel.expr import eval_pb2 as cel_dot_expr_dot_eval__pb2
from cel.expr import syntax_pb2 as cel_dot_expr_dot_syntax__pb2
from google.rpc import status_pb2 as google_dot_rpc_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.cel/expr/conformance/conformance_service.proto\x12\x14\x63\x65l.expr.conformance\x1a\x16\x63\x65l/expr/checked.proto\x1a\x13\x63\x65l/expr/eval.proto\x1a\x15\x63\x65l/expr/syntax.proto\x1a\x17google/rpc/status.proto\"k\n\x0cParseRequest\x12\x12\n\ncel_source\x18\x01 \x01(\t\x12\x16\n\x0esyntax_version\x18\x02 \x01(\t\x12\x17\n\x0fsource_location\x18\x03 \x01(\t\x12\x16\n\x0e\x64isable_macros\x18\x04 \x01(\x08\"^\n\rParseResponse\x12)\n\x0bparsed_expr\x18\x01 \x01(\x0b\x32\x14.cel.expr.ParsedExpr\x12\"\n\x06issues\x18\x02 \x03(\x0b\x32\x12.google.rpc.Status\"\x82\x01\n\x0c\x43heckRequest\x12)\n\x0bparsed_expr\x18\x01 \x01(\x0b\x32\x14.cel.expr.ParsedExpr\x12 \n\x08type_env\x18\x02 \x03(\x0b\x32\x0e.cel.expr.Decl\x12\x11\n\tcontainer\x18\x03 \x01(\t\x12\x12\n\nno_std_env\x18\x04 \x01(\x08\"`\n\rCheckResponse\x12+\n\x0c\x63hecked_expr\x18\x01 \x01(\x0b\x32\x15.cel.expr.CheckedExpr\x12\"\n\x06issues\x18\x02 \x03(\x0b\x32\x12.google.rpc.Status\"\x92\x02\n\x0b\x45valRequest\x12+\n\x0bparsed_expr\x18\x01 \x01(\x0b\x32\x14.cel.expr.ParsedExprH\x00\x12-\n\x0c\x63hecked_expr\x18\x02 \x01(\x0b\x32\x15.cel.expr.CheckedExprH\x00\x12\x41\n\x08\x62indings\x18\x03 \x03(\x0b\x32/.cel.expr.conformance.EvalRequest.BindingsEntry\x12\x11\n\tcontainer\x18\x04 \x01(\t\x1a\x44\n\rBindingsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.cel.expr.ExprValue:\x02\x38\x01\x42\x0b\n\texpr_kind\"W\n\x0c\x45valResponse\x12#\n\x06result\x18\x01 \x01(\x0b\x32\x13.cel.expr.ExprValue\x12\"\n\x06issues\x18\x02 \x03(\x0b\x32\x12.google.rpc.Status\"P\n\x0eSourcePosition\x12\x10\n\x08location\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\x0c\n\x04line\x18\x03 \x01(\x05\x12\x0e\n\x06\x63olumn\x18\x04 \x01(\x05\"\xe0\x01\n\x0cIssueDetails\x12=\n\x08severity\x18\x01 \x01(\x0e\x32+.cel.expr.conformance.IssueDetails.Severity\x12\x36\n\x08position\x18\x02 \x01(\x0b\x32$.cel.expr.conformance.SourcePosition\x12\n\n\x02id\x18\x03 \x01(\x03\"M\n\x08Severity\x12\x18\n\x14SEVERITY_UNSPECIFIED\x10\x00\x12\x0f\n\x0b\x44\x45PRECATION\x10\x01\x12\x0b\n\x07WARNING\x10\x02\x12\t\n\x05\x45RROR\x10\x03\x32\x8d\x02\n\x12\x43onformanceService\x12R\n\x05Parse\x12\".cel.expr.conformance.ParseRequest\x1a#.cel.expr.conformance.ParseResponse\"\x00\x12R\n\x05\x43heck\x12\".cel.expr.conformance.CheckRequest\x1a#.cel.expr.conformance.CheckResponse\"\x00\x12O\n\x04\x45val\x12!.cel.expr.conformance.EvalRequest\x1a\".cel.expr.conformance.EvalResponse\"\x00\x42R\n\x18\x64\x65v.cel.expr.conformanceB\x17\x43onformanceServiceProtoP\x01Z\x18\x63\x65l.dev/expr/conformance\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cel.expr.conformance.conformance_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030dev.cel.expr.conformanceB\027ConformanceServiceProtoP\001Z\030cel.dev/expr/conformance\370\001\001'
  _globals['_EVALREQUEST_BINDINGSENTRY']._loaded_options = None
  _globals['_EVALREQUEST_BINDINGSENTRY']._serialized_options = b'8\001'
  _globals['_PARSEREQUEST']._serialized_start=165
  _globals['_PARSEREQUEST']._serialized_end=272
  _globals['_PARSERESPONSE']._serialized_start=274
  _globals['_PARSERESPONSE']._serialized_end=368
  _globals['_CHECKREQUEST']._serialized_start=371
  _globals['_CHECKREQUEST']._serialized_end=501
  _globals['_CHECKRESPONSE']._serialized_start=503
  _globals['_CHECKRESPONSE']._serialized_end=599
  _globals['_EVALREQUEST']._serialized_start=602
  _globals['_EVALREQUEST']._serialized_end=876
  _globals['_EVALREQUEST_BINDINGSENTRY']._serialized_start=795
  _globals['_EVALREQUEST_BINDINGSENTRY']._serialized_end=863
  _globals['_EVALRESPONSE']._serialized_start=878
  _globals['_EVALRESPONSE']._serialized_end=965
  _globals['_SOURCEPOSITION']._serialized_start=967
  _globals['_SOURCEPOSITION']._serialized_end=1047
  _globals['_ISSUEDETAILS']._serialized_start=1050
  _globals['_ISSUEDETAILS']._serialized_end=1274
  _globals['_ISSUEDETAILS_SEVERITY']._serialized_start=1197
  _globals['_ISSUEDETAILS_SEVERITY']._serialized_end=1274
  _globals['_CONFORMANCESERVICE']._serialized_start=1277
  _globals['_CONFORMANCESERVICE']._serialized_end=1546
# @@protoc_insertion_point(module_scope)



# Copyright 2024 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import defaultdict
import datetime
import logging
import os
import sys
import time
from typing import Any, AnyStr, Callable, Dict, List, Optional, Set
import unittest

from google.protobuf import struct_pb2
import grpc_observability
from grpc_observability import _open_telemetry_measures
from grpc_observability._open_telemetry_plugin import OpenTelemetryLabelInjector
from grpc_observability._open_telemetry_plugin import OpenTelemetryPluginOption
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import AggregationTemporality
from opentelemetry.sdk.metrics.export import MetricExportResult
from opentelemetry.sdk.metrics.export import MetricExporter
from opentelemetry.sdk.metrics.export import MetricsData
from opentelemetry.sdk.metrics.export import PeriodicExportingMetricReader

from tests.observability import _test_server

logger = logging.getLogger(__name__)

STREAM_LENGTH = 5
OTEL_EXPORT_INTERVAL_S = 0.5
CSM_METADATA_EXCHANGE_LABEL_KEY = "exchange_labels_key"

# The following metrics should have optional labels when optional
# labels is enabled through OpenTelemetryPlugin.
METRIC_NAME_WITH_OPTIONAL_LABEL = [
    "grpc.client.attempt.duration"
    "grpc.client.attempt.sent_total_compressed_message_size",
    "grpc.client.attempt.rcvd_total_compressed_message_size",
]
CSM_OPTIONAL_LABEL_KEYS = ["csm.service_name", "csm.service_namespace_name"]

# The following metrics should have metadata exchange labels when metadata
# exchange flow is triggered.
METRIC_NAME_WITH_EXCHANGE_LABEL = [
    "grpc.client.attempt.duration"
    "grpc.client.attempt.sent_total_compressed_message_size",
    "grpc.client.attempt.rcvd_total_compressed_message_size",
    "grpc.server.call.duration",
    "grpc.server.call.sent_total_compressed_message_size",
    "grpc.server.call.rcvd_total_compressed_message_size",
]


class OTelMetricExporter(MetricExporter):
    """Implementation of :class:`MetricExporter` that export metrics to the
    provided metric_list.

    all_metrics: A dict which key is grpc_observability._opentelemetry_measures.Metric.name,
        value is a list of labels recorded for that metric.
        An example item of this dict:
            {"grpc.client.attempt.started":
              [{'grpc.method': 'test/UnaryUnary', 'grpc.target': 'localhost:42517'},
               {'grpc.method': 'other', 'grpc.target': 'localhost:42517'}]}
    """

    def __init__(
        self,
        all_metrics: Dict[str, List],
        preferred_temporality: Dict[type, AggregationTemporality] = None,
        preferred_aggregation: Dict[
            type, "opentelemetry.sdk.metrics.view.Aggregation"
        ] = None,
    ):
        super().__init__(
            preferred_temporality=preferred_temporality,
            preferred_aggregation=preferred_aggregation,
        )
        self.all_metrics = all_metrics

    def export(
        self,
        metrics_data: MetricsData,
        timeout_millis: float = 10_000,
        **kwargs,
    ) -> MetricExportResult:
        self.record_metric(metrics_data)
        return MetricExportResult.SUCCESS

    def shutdown(self, timeout_millis: float = 30_000, **kwargs) -> None:
        pass

    def force_flush(self, timeout_millis: float = 10_000) -> bool:
        return True

    def record_metric(self, metrics_data: MetricsData) -> None:
        for resource_metric in metrics_data.resource_metrics:
            for scope_metric in resource_metric.scope_metrics:
                for metric in scope_metric.metrics:
                    for data_point in metric.data.data_points:
                        self.all_metrics[metric.name].append(
                            data_point.attributes
                        )


class TestLabelInjector(OpenTelemetryLabelInjector):
    _exchange_labels: Dict[str, AnyStr]
    _local_labels: Dict[str, str]

    def __init__(
        self, local_labels: Dict[str, str], exchange_labels: Dict[str, str]
    ):
        self._exchange_labels = exchange_labels
        self._local_labels = local_labels

    def get_labels_for_exchange(self) -> Dict[str, AnyStr]:
        return self._exchange_labels

    def get_additional_labels(
        self, include_exchange_labels: bool
    ) -> Dict[str, str]:
        return self._local_labels

    def deserialize_labels(
        self, labels: Dict[str, AnyStr]
    ) -> Dict[str, AnyStr]:
        deserialized_labels = {}
        for key, value in labels.items():
            if "XEnvoyPeerMetadata" == key:
                struct = struct_pb2.Struct()
                struct.ParseFromString(value)

                exchange_labels_value = self._get_value_from_struct(
                    CSM_METADATA_EXCHANGE_LABEL_KEY, struct
                )
                deserialized_labels[CSM_METADATA_EXCHANGE_LABEL_KEY] = (
                    exchange_labels_value
                )
            else:
                deserialized_labels[key] = value

        return deserialized_labels

    def _get_value_from_struct(
        self, key: str, struct: struct_pb2.Struct
    ) -> str:
        value = struct.fields.get(key)
        if not value:
            return "unknown"
        return value.string_value


class TestOpenTelemetryPluginOption(OpenTelemetryPluginOption):
    _label_injector: OpenTelemetryLabelInjector
    _active_on_client: bool
    _active_on_server: bool

    def __init__(
        self,
        label_injector: OpenTelemetryLabelInjector,
        active_on_client: Optional[bool] = True,
        active_on_server: Optional[bool] = True,
    ):
        self._label_injector = label_injector
        self._active_on_client = active_on_client
        self._active_on_server = active_on_server

    def is_active_on_client_channel(self, target: str) -> bool:
        return self._active_on_client

    def is_active_on_server(self, xds: bool) -> bool:
        return self._active_on_server

    def get_label_injector(self) -> OpenTelemetryLabelInjector:
        return self._label_injector


@unittest.skipIf(
    os.name == "nt" or "darwin" in sys.platform,
    "Observability is not supported in Windows and MacOS",
)
class ObservabilityPluginTest(unittest.TestCase):
    def setUp(self):
        self.all_metrics = defaultdict(list)
        otel_exporter = OTelMetricExporter(self.all_metrics)
        reader = PeriodicExportingMetricReader(
            exporter=otel_exporter,
            export_interval_millis=OTEL_EXPORT_INTERVAL_S * 1000,
        )
        self._provider = MeterProvider(metric_readers=[reader])
        self._server = None
        self._port = None

    def tearDown(self):
        if self._server:
            self._server.stop(0)

    def testLabelInjectorWithLocalLabels(self):
        """Local labels in label injector should be added to all metrics."""
        label_injector = TestLabelInjector(
            local_labels={"local_labels_key": "local_labels_value"},
            exchange_labels={},
        )
        plugin_option = TestOpenTelemetryPluginOption(
            label_injector=label_injector
        )
        otel_plugin = grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider, plugin_options=[plugin_option]
        )

        otel_plugin.register_global()
        self._server, port = _test_server.start_server()
        _test_server.unary_unary_call(port=port)
        otel_plugin.deregister_global()

        self._validate_metrics_exist(self.all_metrics)
        for name, label_list in self.all_metrics.items():
            self._validate_label_exist(name, label_list, ["local_labels_key"])

    def testClientSidePluginOption(self):
        label_injector = TestLabelInjector(
            local_labels={"local_labels_key": "local_labels_value"},
            exchange_labels={},
        )
        plugin_option = TestOpenTelemetryPluginOption(
            label_injector=label_injector, active_on_server=False
        )
        otel_plugin = grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider, plugin_options=[plugin_option]
        )

        otel_plugin.register_global()
        server, port = _test_server.start_server()
        self._server = server
        _test_server.unary_unary_call(port=port)
        otel_plugin.deregister_global()

        self._validate_metrics_exist(self.all_metrics)
        for name, label_list in self.all_metrics.items():
            if "grpc.client" in name:
                self._validate_label_exist(
                    name, label_list, ["local_labels_key"]
                )
        for name, label_list in self.all_metrics.items():
            if "grpc.server" in name:
                self._validate_label_not_exist(
                    name, label_list, ["local_labels_key"]
                )

    def testServerSidePluginOption(self):
        label_injector = TestLabelInjector(
            local_labels={"local_labels_key": "local_labels_value"},
            exchange_labels={},
        )
        plugin_option = TestOpenTelemetryPluginOption(
            label_injector=label_injector, active_on_client=False
        )
        otel_plugin = grpc_observability.OpenTelemetryPlugin(
            meter_provider=self._provider, plugin_options=[plugin_option]
        )

        otel_plugin.register_global()
        server, port = _test_server.start_server()
        self._server = server
        _test_server.unary_unary_call(port=port)
        otel_plugin.deregister_global()

        self._validate_metrics_exist(self.all_metrics)
        for name, label_list in self.all_metrics.items():
            if "grpc.client" in name:
                self._validate_label_not_exist(
                    name, label_list, ["local_labels_key"]
                )
        for name, label_list in self.all_metrics.items():
            if "grpc.server" in name:
                self._validate_label_exist(
                    name, label_list, ["local_labels_key"]
                )

    def assert_eventually(
        self,
        predicate: Callable[[], bool],
        *,
        timeout: Optional[datetime.timedelta] = None,
        message: Optional[Callable[[], str]] = None,
    ) -> None:
        message = message or (lambda: "Proposition did not evaluate to true")
        timeout = timeout or datetime.timedelta(seconds=5)
        end = datetime.datetime.now() + timeout
        while datetime.datetime.now() < end:
            if predicate():
                break
            time.sleep(0.5)
        else:
            self.fail(message() + " after " + str(timeout))

    def _validate_metrics_exist(self, all_metrics: Dict[str, Any]) -> None:
        # Sleep here to make sure we have at least one export from OTel MetricExporter.
        self.assert_eventually(
            lambda: len(all_metrics.keys()) > 1,
            message=lambda: f"No metrics was exported",
        )

    def _validate_all_metrics_names(self, metric_names: Set[str]) -> None:
        self._validate_server_metrics_names(metric_names)
        self._validate_client_metrics_names(metric_names)

    def _validate_server_metrics_names(self, metric_names: Set[str]) -> None:
        for base_metric in _open_telemetry_measures.base_metrics():
            if "grpc.server" in base_metric.name:
                self.assertTrue(
                    base_metric.name in metric_names,
                    msg=f"metric {base_metric.name} not found in exported metrics: {metric_names}!",
                )

    def _validate_client_metrics_names(self, metric_names: Set[str]) -> None:
        for base_metric in _open_telemetry_measures.base_metrics():
            if "grpc.client" in base_metric.name:
                self.assertTrue(
                    base_metric.name in metric_names,
                    msg=f"metric {base_metric.name} not found in exported metrics: {metric_names}!",
                )

    def _validate_label_exist(
        self,
        metric_name: str,
        metric_label_list: List[str],
        labels_to_check: List[str],
    ) -> None:
        for metric_label in metric_label_list:
            for label in labels_to_check:
                self.assertTrue(
                    label in metric_label,
                    msg=f"label with key {label} not found in metric {metric_name}, found label list: {metric_label}",
                )

    def _validate_label_not_exist(
        self,
        metric_name: str,
        metric_label_list: List[str],
        labels_to_check: List[str],
    ) -> None:
        for metric_label in metric_label_list:
            for label in labels_to_check:
                self.assertFalse(
                    label in metric_label,
                    msg=f"found unexpected label with key {label} in metric {metric_name}, found label list: {metric_label}",
                )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Test of RPCs made against gRPC Python's application-layer API."""

from concurrent import futures
import itertools
import logging
import threading
import unittest

import grpc
from grpc.framework.foundation import logging_pool

from tests.unit._rpc_test_helpers import (
    stream_stream_non_blocking_multi_callable,
)
from tests.unit._rpc_test_helpers import (
    unary_stream_non_blocking_multi_callable,
)
from tests.unit._rpc_test_helpers import BaseRPCTest
from tests.unit._rpc_test_helpers import Callback
from tests.unit._rpc_test_helpers import TIMEOUT_SHORT
from tests.unit._rpc_test_helpers import stream_stream_multi_callable
from tests.unit._rpc_test_helpers import stream_unary_multi_callable
from tests.unit._rpc_test_helpers import unary_stream_multi_callable
from tests.unit._rpc_test_helpers import unary_unary_multi_callable
from tests.unit.framework.common import test_constants


class RPCPart1Test(BaseRPCTest, unittest.TestCase):
    def testExpiredStreamRequestBlockingUnaryResponse(self):
        requests = tuple(
            b"\x07\x08" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)

        multi_callable = stream_unary_multi_callable(self._channel)
        with self._control.pause():
            with self.assertRaises(grpc.RpcError) as exception_context:
                multi_callable(
                    request_iterator,
                    timeout=TIMEOUT_SHORT,
                    metadata=(
                        ("test", "ExpiredStreamRequestBlockingUnaryResponse"),
                    ),
                )

        self.assertIsInstance(exception_context.exception, grpc.RpcError)
        self.assertIsInstance(exception_context.exception, grpc.Call)
        self.assertIsNotNone(exception_context.exception.initial_metadata())
        self.assertIs(
            grpc.StatusCode.DEADLINE_EXCEEDED,
            exception_context.exception.code(),
        )
        self.assertIsNotNone(exception_context.exception.details())
        self.assertIsNotNone(exception_context.exception.trailing_metadata())

    def testExpiredStreamRequestFutureUnaryResponse(self):
        requests = tuple(
            b"\x07\x18" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)
        callback = Callback()

        multi_callable = stream_unary_multi_callable(self._channel)
        with self._control.pause():
            response_future = multi_callable.future(
                request_iterator,
                timeout=TIMEOUT_SHORT,
                metadata=(("test", "ExpiredStreamRequestFutureUnaryResponse"),),
            )
            with self.assertRaises(grpc.FutureTimeoutError):
                response_future.result(timeout=TIMEOUT_SHORT / 2.0)
            response_future.add_done_callback(callback)
            value_passed_to_callback = callback.value()

        with self.assertRaises(grpc.RpcError) as exception_context:
            response_future.result()
        self.assertIs(grpc.StatusCode.DEADLINE_EXCEEDED, response_future.code())
        self.assertIs(
            grpc.StatusCode.DEADLINE_EXCEEDED,
            exception_context.exception.code(),
        )
        self.assertIsInstance(response_future.exception(), grpc.RpcError)
        self.assertIsNotNone(response_future.traceback())
        self.assertIs(response_future, value_passed_to_callback)
        self.assertIsNotNone(response_future.initial_metadata())
        self.assertIs(grpc.StatusCode.DEADLINE_EXCEEDED, response_future.code())
        self.assertIsNotNone(response_future.details())
        self.assertIsNotNone(response_future.trailing_metadata())

    def testExpiredStreamRequestStreamResponse(self):
        self._expired_stream_request_stream_response(
            stream_stream_multi_callable(self._channel)
        )

    def testExpiredStreamRequestStreamResponseNonBlocking(self):
        self._expired_stream_request_stream_response(
            stream_stream_non_blocking_multi_callable(self._channel)
        )

    def testFailedUnaryRequestBlockingUnaryResponse(self):
        request = b"\x37\x17"

        multi_callable = unary_unary_multi_callable(self._channel)
        with self._control.fail():
            with self.assertRaises(grpc.RpcError) as exception_context:
                multi_callable.with_call(
                    request,
                    metadata=(
                        ("test", "FailedUnaryRequestBlockingUnaryResponse"),
                    ),
                )

        self.assertIs(
            grpc.StatusCode.UNKNOWN, exception_context.exception.code()
        )
        # sanity checks on to make sure returned string contains default members
        # of the error
        debug_error_string = exception_context.exception.debug_error_string()
        self.assertIn("Exception calling application", debug_error_string)

    def testFailedUnaryRequestFutureUnaryResponse(self):
        request = b"\x37\x17"
        callback = Callback()

        multi_callable = unary_unary_multi_callable(self._channel)
        with self._control.fail():
            response_future = multi_callable.future(
                request,
                metadata=(("test", "FailedUnaryRequestFutureUnaryResponse"),),
            )
            response_future.add_done_callback(callback)
            value_passed_to_callback = callback.value()

        self.assertIsInstance(response_future, grpc.Future)
        self.assertIsInstance(response_future, grpc.Call)
        with self.assertRaises(grpc.RpcError) as exception_context:
            response_future.result()
        self.assertIs(
            grpc.StatusCode.UNKNOWN, exception_context.exception.code()
        )
        self.assertIsInstance(response_future.exception(), grpc.RpcError)
        self.assertIsNotNone(response_future.traceback())
        self.assertIs(
            grpc.StatusCode.UNKNOWN, response_future.exception().code()
        )
        self.assertIs(response_future, value_passed_to_callback)

    def testFailedUnaryRequestStreamResponse(self):
        self._failed_unary_request_stream_response(
            unary_stream_multi_callable(self._channel)
        )

    def testFailedUnaryRequestStreamResponseNonBlocking(self):
        self._failed_unary_request_stream_response(
            unary_stream_non_blocking_multi_callable(self._channel)
        )

    def testFailedStreamRequestBlockingUnaryResponse(self):
        requests = tuple(
            b"\x47\x58" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)

        multi_callable = stream_unary_multi_callable(self._channel)
        with self._control.fail():
            with self.assertRaises(grpc.RpcError) as exception_context:
                multi_callable(
                    request_iterator,
                    metadata=(
                        ("test", "FailedStreamRequestBlockingUnaryResponse"),
                    ),
                )

        self.assertIs(
            grpc.StatusCode.UNKNOWN, exception_context.exception.code()
        )

    def testFailedStreamRequestFutureUnaryResponse(self):
        requests = tuple(
            b"\x07\x18" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)
        callback = Callback()

        multi_callable = stream_unary_multi_callable(self._channel)
        with self._control.fail():
            response_future = multi_callable.future(
                request_iterator,
                metadata=(("test", "FailedStreamRequestFutureUnaryResponse"),),
            )
            response_future.add_done_callback(callback)
            value_passed_to_callback = callback.value()

        with self.assertRaises(grpc.RpcError) as exception_context:
            response_future.result()
        self.assertIs(grpc.StatusCode.UNKNOWN, response_future.code())
        self.assertIs(
            grpc.StatusCode.UNKNOWN, exception_context.exception.code()
        )
        self.assertIsInstance(response_future.exception(), grpc.RpcError)
        self.assertIsNotNone(response_future.traceback())
        self.assertIs(response_future, value_passed_to_callback)

    def testFailedStreamRequestStreamResponse(self):
        self._failed_stream_request_stream_response(
            stream_stream_multi_callable(self._channel)
        )

    def testFailedStreamRequestStreamResponseNonBlocking(self):
        self._failed_stream_request_stream_response(
            stream_stream_non_blocking_multi_callable(self._channel)
        )

    def testIgnoredUnaryRequestFutureUnaryResponse(self):
        request = b"\x37\x17"

        multi_callable = unary_unary_multi_callable(self._channel)
        multi_callable.future(
            request,
            metadata=(("test", "IgnoredUnaryRequestFutureUnaryResponse"),),
        )

    def testIgnoredUnaryRequestStreamResponse(self):
        self._ignored_unary_stream_request_future_unary_response(
            unary_stream_multi_callable(self._channel)
        )

    def testIgnoredUnaryRequestStreamResponseNonBlocking(self):
        self._ignored_unary_stream_request_future_unary_response(
            unary_stream_non_blocking_multi_callable(self._channel)
        )

    def testIgnoredStreamRequestFutureUnaryResponse(self):
        requests = tuple(
            b"\x07\x18" for _ in range(test_constants.STREAM_LENGTH)
        )
        request_iterator = iter(requests)

        multi_callable = stream_unary_multi_callable(self._channel)
        multi_callable.future(
            request_iterator,
            metadata=(("test", "IgnoredStreamRequestFutureUnaryResponse"),),
        )

    def testIgnoredStreamRequestStreamResponse(self):
        self._ignored_stream_request_stream_response(
            stream_stream_multi_callable(self._channel)
        )

    def testIgnoredStreamRequestStreamResponseNonBlocking(self):
        self._ignored_stream_request_stream_response(
            stream_stream_non_blocking_multi_callable(self._channel)
        )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=3)



# Copyright 2019 The gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests behavior of the connectivity state."""

import asyncio
import logging
import platform
import threading
import time
import unittest

import grpc
from grpc.experimental import aio

from tests.unit.framework.common import test_constants
from tests_aio.unit import _common
from tests_aio.unit._constants import UNREACHABLE_TARGET
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server


class TestConnectivityState(AioTestBase):
    async def setUp(self):
        self._server_address, self._server = await start_test_server()

    async def tearDown(self):
        await self._server.stop(None)

    @unittest.skipIf(
        "aarch64" in platform.machine(),
        "The transient failure propagation is slower on aarch64",
    )
    async def test_unavailable_backend(self):
        async with aio.insecure_channel(UNREACHABLE_TARGET) as channel:
            self.assertEqual(
                grpc.ChannelConnectivity.IDLE, channel.get_state(False)
            )
            self.assertEqual(
                grpc.ChannelConnectivity.IDLE, channel.get_state(True)
            )

            # Should not time out
            await asyncio.wait_for(
                _common.block_until_certain_state(
                    channel, grpc.ChannelConnectivity.TRANSIENT_FAILURE
                ),
                test_constants.SHORT_TIMEOUT * 2,
            )

    async def test_normal_backend(self):
        async with aio.insecure_channel(self._server_address) as channel:
            current_state = channel.get_state(True)
            self.assertEqual(grpc.ChannelConnectivity.IDLE, current_state)

            # Should not time out
            await asyncio.wait_for(
                _common.block_until_certain_state(
                    channel, grpc.ChannelConnectivity.READY
                ),
                test_constants.SHORT_TIMEOUT,
            )

    async def test_timeout(self):
        async with aio.insecure_channel(self._server_address) as channel:
            self.assertEqual(
                grpc.ChannelConnectivity.IDLE, channel.get_state(False)
            )

            # If timed out, the function should return None.
            with self.assertRaises(asyncio.TimeoutError):
                await asyncio.wait_for(
                    _common.block_until_certain_state(
                        channel, grpc.ChannelConnectivity.READY
                    ),
                    test_constants.SHORT_TIMEOUT,
                )

    async def test_shutdown(self):
        channel = aio.insecure_channel(self._server_address)

        self.assertEqual(
            grpc.ChannelConnectivity.IDLE, channel.get_state(False)
        )

        # Waiting for changes in a separate coroutine
        wait_started = asyncio.Event()

        async def a_pending_wait():
            wait_started.set()
            await channel.wait_for_state_change(grpc.ChannelConnectivity.IDLE)

        pending_task = self.loop.create_task(a_pending_wait())
        await wait_started.wait()

        await channel.close()

        self.assertEqual(
            grpc.ChannelConnectivity.SHUTDOWN, channel.get_state(True)
        )

        self.assertEqual(
            grpc.ChannelConnectivity.SHUTDOWN, channel.get_state(False)
        )

        # Make sure there isn't any exception in the task
        await pending_task

        # It can raise exceptions since it is a usage error, but it should not
        # segfault or abort.
        with self.assertRaises(aio.UsageError):
            await channel.wait_for_state_change(
                grpc.ChannelConnectivity.SHUTDOWN
            )


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import grpc_testing


class _UnaryUnary(grpc_testing.UnaryUnaryChannelRpc):
    def __init__(self, rpc_state):
        self._rpc_state = rpc_state

    def send_initial_metadata(self, initial_metadata):
        self._rpc_state.send_initial_metadata(initial_metadata)

    def cancelled(self):
        self._rpc_state.cancelled()

    def terminate(self, response, trailing_metadata, code, details):
        self._rpc_state.terminate_with_response(
            response, trailing_metadata, code, details
        )


class _UnaryStream(grpc_testing.UnaryStreamChannelRpc):
    def __init__(self, rpc_state):
        self._rpc_state = rpc_state

    def send_initial_metadata(self, initial_metadata):
        self._rpc_state.send_initial_metadata(initial_metadata)

    def send_response(self, response):
        self._rpc_state.send_response(response)

    def cancelled(self):
        self._rpc_state.cancelled()

    def terminate(self, trailing_metadata, code, details):
        self._rpc_state.terminate(trailing_metadata, code, details)


class _StreamUnary(grpc_testing.StreamUnaryChannelRpc):
    def __init__(self, rpc_state):
        self._rpc_state = rpc_state

    def send_initial_metadata(self, initial_metadata):
        self._rpc_state.send_initial_metadata(initial_metadata)

    def take_request(self):
        return self._rpc_state.take_request()

    def requests_closed(self):
        return self._rpc_state.requests_closed()

    def cancelled(self):
        self._rpc_state.cancelled()

    def terminate(self, response, trailing_metadata, code, details):
        self._rpc_state.terminate_with_response(
            response, trailing_metadata, code, details
        )


class _StreamStream(grpc_testing.StreamStreamChannelRpc):
    def __init__(self, rpc_state):
        self._rpc_state = rpc_state

    def send_initial_metadata(self, initial_metadata):
        self._rpc_state.send_initial_metadata(initial_metadata)

    def take_request(self):
        return self._rpc_state.take_request()

    def send_response(self, response):
        self._rpc_state.send_response(response)

    def requests_closed(self):
        return self._rpc_state.requests_closed()

    def cancelled(self):
        self._rpc_state.cancelled()

    def terminate(self, trailing_metadata, code, details):
        self._rpc_state.terminate(trailing_metadata, code, details)


def unary_unary(channel_state, method_descriptor):
    rpc_state = channel_state.take_rpc_state(method_descriptor)
    (
        invocation_metadata,
        request,
    ) = rpc_state.take_invocation_metadata_and_request()
    return invocation_metadata, request, _UnaryUnary(rpc_state)


def unary_stream(channel_state, method_descriptor):
    rpc_state = channel_state.take_rpc_state(method_descriptor)
    (
        invocation_metadata,
        request,
    ) = rpc_state.take_invocation_metadata_and_request()
    return invocation_metadata, request, _UnaryStream(rpc_state)


def stream_unary(channel_state, method_descriptor):
    rpc_state = channel_state.take_rpc_state(method_descriptor)
    return rpc_state.take_invocation_metadata(), _StreamUnary(rpc_state)


def stream_stream(channel_state, method_descriptor):
    rpc_state = channel_state.take_rpc_state(method_descriptor)
    return rpc_state.take_invocation_metadata(), _StreamStream(rpc_state)



# Copyright 2018 gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Provides setuptools command classes for the GRPC Python setup process."""

import os
import shutil

import setuptools

ROOT_DIR = os.path.abspath(os.path.dirname(os.path.abspath(__file__)))
GRPC_ROOT_ABS_PATH = os.path.join(ROOT_DIR, "../../..")
ROOT_REL_DIR = os.path.relpath(ROOT_DIR, start=GRPC_ROOT_ABS_PATH)
LICENSE = "./LICENSE"


class Preprocess(setuptools.Command):
    """Command to copy LICENSE from root directory."""

    description = ""
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if os.path.isfile(LICENSE):
            shutil.copyfile(LICENSE, os.path.join(ROOT_REL_DIR, "LICENSE"))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/http/ext_proc/processing_request_modifiers/mapped_attribute_builder/v3/mapped_attribute_builder.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/http/ext_proc/processing_request_modifiers/mapped_attribute_builder/v3/mapped_attribute_builder.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nvenvoy/extensions/http/ext_proc/processing_request_modifiers/mapped_attribute_builder/v3/mapped_attribute_builder.proto\x12Wenvoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\x81\x04\n\x16MappedAttributeBuilder\x12\xb0\x01\n\x19mapped_request_attributes\x18\x01 \x03(\x0b\x32\x8c\x01.envoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3.MappedAttributeBuilder.MappedRequestAttributesEntry\x12\xb2\x01\n\x1amapped_response_attributes\x18\x02 \x03(\x0b\x32\x8d\x01.envoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3.MappedAttributeBuilder.MappedResponseAttributesEntry\x1a>\n\x1cMappedRequestAttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a?\n\x1dMappedResponseAttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\xb2\x02\neio.envoyproxy.envoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3B\x1bMappedAttributeBuilderProtoP\x01Z\x99\x01github.com/envoyproxy/go-control-plane/envoy/extensions/http/ext_proc/processing_request_modifiers/mapped_attribute_builder/v3;mapped_attribute_builderv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3.mapped_attribute_builder_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\neio.envoyproxy.envoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3B\033MappedAttributeBuilderProtoP\001Z\231\001github.com/envoyproxy/go-control-plane/envoy/extensions/http/ext_proc/processing_request_modifiers/mapped_attribute_builder/v3;mapped_attribute_builderv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDREQUESTATTRIBUTESENTRY']._loaded_options = None
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDREQUESTATTRIBUTESENTRY']._serialized_options = b'8\001'
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDRESPONSEATTRIBUTESENTRY']._loaded_options = None
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDRESPONSEATTRIBUTESENTRY']._serialized_options = b'8\001'
  _globals['_MAPPEDATTRIBUTEBUILDER']._serialized_start=276
  _globals['_MAPPEDATTRIBUTEBUILDER']._serialized_end=789
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDREQUESTATTRIBUTESENTRY']._serialized_start=662
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDREQUESTATTRIBUTESENTRY']._serialized_end=724
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDRESPONSEATTRIBUTESENTRY']._serialized_start=726
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDRESPONSEATTRIBUTESENTRY']._serialized_end=789
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cel/expr/explain.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cel/expr/explain.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cel.expr import value_pb2 as cel_dot_expr_dot_value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16\x63\x65l/expr/explain.proto\x12\x08\x63\x65l.expr\x1a\x14\x63\x65l/expr/value.proto\"\x8b\x01\n\x07\x45xplain\x12\x1f\n\x06values\x18\x01 \x03(\x0b\x32\x0f.cel.expr.Value\x12.\n\nexpr_steps\x18\x02 \x03(\x0b\x32\x1a.cel.expr.Explain.ExprStep\x1a+\n\x08\x45xprStep\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x13\n\x0bvalue_index\x18\x02 \x01(\x05:\x02\x18\x01\x42/\n\x0c\x64\x65v.cel.exprB\x0c\x45xplainProtoP\x01Z\x0c\x63\x65l.dev/expr\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cel.expr.explain_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\014dev.cel.exprB\014ExplainProtoP\001Z\014cel.dev/expr\370\001\001'
  _globals['_EXPLAIN']._loaded_options = None
  _globals['_EXPLAIN']._serialized_options = b'\030\001'
  _globals['_EXPLAIN']._serialized_start=59
  _globals['_EXPLAIN']._serialized_end=198
  _globals['_EXPLAIN_EXPRSTEP']._serialized_start=151
  _globals['_EXPLAIN_EXPRSTEP']._serialized_end=194
# @@protoc_insertion_point(module_scope)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Defines test client behaviors (UNARY/STREAMING) (SYNC/ASYNC)."""

import abc
from concurrent import futures
import queue
import threading
import time

import grpc

from src.proto.grpc.testing import benchmark_service_pb2_grpc
from src.proto.grpc.testing import messages_pb2
from tests.unit import resources
from tests.unit import test_common

_TIMEOUT = 60 * 60 * 24


class GenericStub:
    def __init__(self, channel):
        self.UnaryCall = channel.unary_unary(
            "/grpc.testing.BenchmarkService/UnaryCall",
            _registered_method=True,
        )
        self.StreamingFromServer = channel.unary_stream(
            "/grpc.testing.BenchmarkService/StreamingFromServer",
            _registered_method=True,
        )
        self.StreamingCall = channel.stream_stream(
            "/grpc.testing.BenchmarkService/StreamingCall",
            _registered_method=True,
        )


class BenchmarkClient:
    """Benchmark client interface that exposes a non-blocking send_request()."""

    __metaclass__ = abc.ABCMeta

    def __init__(self, server, config, hist):
        # Create the stub
        if config.HasField("security_params"):
            creds = grpc.ssl_channel_credentials(
                resources.test_root_certificates()
            )
            channel = test_common.test_secure_channel(
                server, creds, config.security_params.server_host_override
            )
        else:
            channel = grpc.insecure_channel(server)

        # waits for the channel to be ready before we start sending messages
        grpc.channel_ready_future(channel).result()

        if config.payload_config.WhichOneof("payload") == "simple_params":
            self._generic = False
            self._stub = benchmark_service_pb2_grpc.BenchmarkServiceStub(
                channel
            )
            payload = messages_pb2.Payload(
                body=bytes(b"\0" * config.payload_config.simple_params.req_size)
            )
            self._request = messages_pb2.SimpleRequest(
                payload=payload,
                response_size=config.payload_config.simple_params.resp_size,
            )
        else:
            self._generic = True
            self._stub = GenericStub(channel)
            self._request = bytes(
                b"\0" * config.payload_config.bytebuf_params.req_size
            )

        self._hist = hist
        self._response_callbacks = []

    def add_response_callback(self, callback):
        """callback will be invoked as callback(client, query_time)"""
        self._response_callbacks.append(callback)

    @abc.abstractmethod
    def send_request(self):
        """Non-blocking wrapper for a client's request operation."""
        raise NotImplementedError()

    def start(self):
        pass

    def stop(self):
        pass

    def _handle_response(self, client, query_time):
        self._hist.add(query_time * 1e9)  # Report times in nanoseconds
        for callback in self._response_callbacks:
            callback(client, query_time)


class UnarySyncBenchmarkClient(BenchmarkClient):
    def __init__(self, server, config, hist):
        super(UnarySyncBenchmarkClient, self).__init__(server, config, hist)
        self._pool = futures.ThreadPoolExecutor(
            max_workers=config.outstanding_rpcs_per_channel
        )

    def send_request(self):
        # Send requests in separate threads to support multiple outstanding rpcs
        # (See src/proto/grpc/testing/control.proto)
        self._pool.submit(self._dispatch_request)

    def stop(self):
        self._pool.shutdown(wait=True)
        self._stub = None

    def _dispatch_request(self):
        start_time = time.time()
        self._stub.UnaryCall(self._request, _TIMEOUT)
        end_time = time.time()
        self._handle_response(self, end_time - start_time)


class UnaryAsyncBenchmarkClient(BenchmarkClient):
    def send_request(self):
        # Use the Future callback api to support multiple outstanding rpcs
        start_time = time.time()
        response_future = self._stub.UnaryCall.future(self._request, _TIMEOUT)
        response_future.add_done_callback(
            lambda resp: self._response_received(start_time, resp)
        )

    def _response_received(self, start_time, resp):
        resp.result()
        end_time = time.time()
        self._handle_response(self, end_time - start_time)

    def stop(self):
        self._stub = None


class _SyncStream:
    def __init__(self, stub, generic, request, handle_response):
        self._stub = stub
        self._generic = generic
        self._request = request
        self._handle_response = handle_response
        self._is_streaming = False
        self._request_queue = queue.Queue()
        self._send_time_queue = queue.Queue()

    def send_request(self):
        self._send_time_queue.put(time.time())
        self._request_queue.put(self._request)

    def start(self):
        self._is_streaming = True
        response_stream = self._stub.StreamingCall(
            self._request_generator(), _TIMEOUT
        )
        for _ in response_stream:
            self._handle_response(
                self, time.time() - self._send_time_queue.get_nowait()
            )

    def stop(self):
        self._is_streaming = False

    def _request_generator(self):
        while self._is_streaming:
            try:
                request = self._request_queue.get(block=True, timeout=1.0)
                yield request
            except queue.Empty:
                pass


class StreamingSyncBenchmarkClient(BenchmarkClient):
    def __init__(self, server, config, hist):
        super(StreamingSyncBenchmarkClient, self).__init__(server, config, hist)
        self._pool = futures.ThreadPoolExecutor(
            max_workers=config.outstanding_rpcs_per_channel
        )
        self._streams = [
            _SyncStream(
                self._stub, self._generic, self._request, self._handle_response
            )
            for _ in range(config.outstanding_rpcs_per_channel)
        ]
        self._curr_stream = 0

    def send_request(self):
        # Use a round_robin scheduler to determine what stream to send on
        self._streams[self._curr_stream].send_request()
        self._curr_stream = (self._curr_stream + 1) % len(self._streams)

    def start(self):
        for stream in self._streams:
            self._pool.submit(stream.start)

    def stop(self):
        for stream in self._streams:
            stream.stop()
        self._pool.shutdown(wait=True)
        self._stub = None


class ServerStreamingSyncBenchmarkClient(BenchmarkClient):
    def __init__(self, server, config, hist):
        super(ServerStreamingSyncBenchmarkClient, self).__init__(
            server, config, hist
        )
        if config.outstanding_rpcs_per_channel == 1:
            self._pool = None
        else:
            self._pool = futures.ThreadPoolExecutor(
                max_workers=config.outstanding_rpcs_per_channel
            )
        self._rpcs = []
        self._sender = None

    def send_request(self):
        if self._pool is None:
            self._sender = threading.Thread(
                target=self._one_stream_streaming_rpc, daemon=True
            )
            self._sender.start()
        else:
            self._pool.submit(self._one_stream_streaming_rpc)

    def _one_stream_streaming_rpc(self):
        response_stream = self._stub.StreamingFromServer(
            self._request, _TIMEOUT
        )
        self._rpcs.append(response_stream)
        start_time = time.time()
        for _ in response_stream:
            self._handle_response(self, time.time() - start_time)
            start_time = time.time()

    def stop(self):
        for call in self._rpcs:
            call.cancel()
        if self._sender is not None:
            self._sender.join()
        if self._pool is not None:
            self._pool.shutdown(wait=False)
        self._stub = None



#!/usr/bin/env python3

# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

os.chdir(os.path.join(os.path.dirname(sys.argv[0]), "../../.."))

# Allowance for overrides for specific files
EXPECTED_NAMES = {
    "src/proto/grpc/channelz": "channelz",
    "src/proto/grpc/status": "status",
    "src/proto/grpc/testing": "testing",
    "src/proto/grpc/testing/duplicate": "duplicate",
    "src/proto/grpc/lb/v1": "lb",
    "src/proto/grpc/testing/xds": "xds",
    "src/proto/grpc/testing/xds/v3": "xds_v3",
    "src/proto/grpc/core": "core",
    "src/proto/grpc/health/v1": "health",
    "src/proto/grpc/reflection/v1alpha": "reflection",
    "src/proto/grpc/reflection/v1": "reflection_v1",
}

errors = 0
for root, dirs, files in os.walk("."):
    if root.startswith("./"):
        root = root[len("./") :]
    # don't check third party
    if root.startswith("third_party/"):
        continue
    # only check BUILD files
    if "BUILD" not in files:
        continue
    text = open("%s/BUILD" % root).read()
    # find a grpc_package clause
    pkg_start = text.find("grpc_package(")
    if pkg_start == -1:
        continue
    # parse it, taking into account nested parens
    pkg_end = pkg_start + len("grpc_package(")
    level = 1
    while level == 1:
        if text[pkg_end] == ")":
            level -= 1
        elif text[pkg_end] == "(":
            level += 1
        pkg_end += 1
    # it's a python statement, so evaluate it to pull out the name of the package
    name = eval(
        text[pkg_start:pkg_end], {"grpc_package": lambda name, **kwargs: name}
    )
    # the name should be the path within the source tree, excepting some special
    # BUILD files (really we should normalize them too at some point)
    # TODO(ctiller): normalize all package names
    expected_name = EXPECTED_NAMES.get(root, root)
    if name != expected_name:
        print(
            "%s/BUILD should define a grpc_package with name=%r, not %r"
            % (root, expected_name, name)
        )
        errors += 1

if errors != 0:
    sys.exit(1)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import time

import http2_base_server


class TestcaseGoaway:
    """
    This test does the following:
      Process incoming request normally, i.e. send headers, data and trailers.
      Then send a GOAWAY frame with the stream id of the processed request.
      It checks that the next request is made on a different TCP connection.
    """

    def __init__(self, iteration):
        self._base_server = http2_base_server.H2ProtocolBaseServer()
        self._base_server._handlers["RequestReceived"] = (
            self.on_request_received
        )
        self._base_server._handlers["DataReceived"] = self.on_data_received
        self._base_server._handlers["SendDone"] = self.on_send_done
        self._base_server._handlers["ConnectionLost"] = self.on_connection_lost
        self._ready_to_send = False
        self._iteration = iteration

    def get_base_server(self):
        return self._base_server

    def on_connection_lost(self, reason):
        logging.info("Disconnect received. Count %d" % self._iteration)
        # _iteration == 2 => Two different connections have been used.
        if self._iteration == 2:
            self._base_server.on_connection_lost(reason)

    def on_send_done(self, stream_id):
        self._base_server.on_send_done_default(stream_id)
        logging.info("Sending GOAWAY for stream %d:" % stream_id)
        self._base_server._conn.close_connection(
            error_code=0, additional_data=None, last_stream_id=stream_id
        )
        self._base_server._stream_status[stream_id] = False

    def on_request_received(self, event):
        self._ready_to_send = False
        self._base_server.on_request_received_default(event)

    def on_data_received(self, event):
        self._base_server.on_data_received_default(event)
        sr = self._base_server.parse_received_data(event.stream_id)
        if sr:
            logging.info("Creating response size = %s" % sr.response_size)
            response_data = self._base_server.default_response_data(
                sr.response_size
            )
            self._ready_to_send = True
            self._base_server.setup_send(response_data, event.stream_id)



#!/usr/bin/env python3

# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import os
import os.path
import re
import subprocess
import sys


def build_valid_guard(fpath):
    guard_components = (
        fpath.replace("++", "XX").replace(".", "_").upper().split("/")
    )
    if fpath.startswith("include/"):
        return "_".join(guard_components[1:])
    else:
        return "GRPC_" + "_".join(guard_components)


def load(fpath):
    with open(fpath, "r") as f:
        return f.read()


def save(fpath, contents):
    with open(fpath, "w") as f:
        f.write(contents)


class GuardValidator:
    def __init__(self):
        self.ifndef_re = re.compile(r"#ifndef ([A-Z][A-Z_0-9]*)")
        self.define_re = re.compile(r"#define ([A-Z][A-Z_0-9]*)")
        self.endif_c_core_re = re.compile(
            r"#endif /\* (?: *\\\n *)?([A-Z][A-Z_0-9]*) (?:\\\n *)?\*/$"
        )
        self.endif_re = re.compile(r"#endif  // ([A-Z][A-Z_0-9]*)")
        self.comments_then_includes_re = re.compile(
            (
                r"^((//.*?$|/\*.*?\*/|[ \r\n\t])*)(([ \r\n\t]|#include"
                r" .*)*)(#ifndef [^\n]*\n#define [^\n]*\n)"
            ),
            re.DOTALL | re.MULTILINE,
        )
        self.failed = False

    def _is_c_core_header(self, fpath):
        return "include" in fpath and not (
            "grpc++" in fpath
            or "grpcpp" in fpath
            or "event_engine" in fpath
            or fpath.endswith("/grpc_audit_logging.h")
            or fpath.endswith("/json.h")
        )

    def fail(self, fpath, regexp, fcontents, match_txt, correct, fix):
        c_core_header = self._is_c_core_header(fpath)
        self.failed = True
        invalid_guards_msg_template = (
            "{0}: Missing preprocessor guards (RE {1}). "
            "Please wrap your code around the following guards:\n"
            "#ifndef {2}\n"
            "#define {2}\n"
            "...\n"
            "... epic code ...\n"
            "...\n"
            + ("#endif /* {2} */" if c_core_header else "#endif  // {2}")
        )
        if not match_txt:
            print(
                (
                    invalid_guards_msg_template.format(
                        fpath, regexp.pattern, build_valid_guard(fpath)
                    )
                )
            )
            return fcontents

        print(
            (
                (
                    "{}: Wrong preprocessor guards (RE {}):"
                    "\n\tFound {}, expected {}"
                ).format(fpath, regexp.pattern, match_txt, correct)
            )
        )
        if fix:
            print("Fixing {}...\n".format(fpath))
            fixed_fcontents = re.sub(match_txt, correct, fcontents)
            if fixed_fcontents:
                self.failed = False
            return fixed_fcontents
        else:
            print()
        return fcontents

    def check(self, fpath, fix):
        c_core_header = self._is_c_core_header(fpath)
        valid_guard = build_valid_guard(fpath)

        fcontents = load(fpath)

        match = self.ifndef_re.search(fcontents)
        if not match:
            print(("something drastically wrong with: %s" % fpath))
            return False  # failed
        if match.lastindex is None:
            # No ifndef. Request manual addition with hints
            self.fail(fpath, match.re, match.string, "", "", False)
            return False  # failed

        # Does the guard end with a '_H'?
        running_guard = match.group(1)
        if not running_guard.endswith("_H"):
            fcontents = self.fail(
                fpath, match.re, match.string, match.group(1), valid_guard, fix
            )
            if fix:
                save(fpath, fcontents)

        # Is it the expected one based on the file path?
        if running_guard != valid_guard:
            fcontents = self.fail(
                fpath, match.re, match.string, match.group(1), valid_guard, fix
            )
            if fix:
                save(fpath, fcontents)

        # Is there a #define? Is it the same as the #ifndef one?
        match = self.define_re.search(fcontents)
        if match.lastindex is None:
            # No define. Request manual addition with hints
            self.fail(fpath, match.re, match.string, "", "", False)
            return False  # failed

        # Is the #define guard the same as the #ifndef guard?
        if match.group(1) != running_guard:
            fcontents = self.fail(
                fpath, match.re, match.string, match.group(1), valid_guard, fix
            )
            if fix:
                save(fpath, fcontents)

        # Is there a properly commented #endif?
        flines = fcontents.rstrip().splitlines()
        # Use findall and use the last result if there are multiple matches,
        # i.e. nested include guards.
        match = self.endif_c_core_re.findall("\n".join(flines[-3:]))
        if not match and not c_core_header:
            match = self.endif_re.findall("\n".join(flines[-3:]))
        if not match:
            # No endif. Check if we have the last line as just '#endif' and if so
            # replace it with a properly commented one.
            if flines[-1] == "#endif":
                flines[-1] = "#endif" + (
                    " /* {} */\n".format(valid_guard)
                    if c_core_header
                    else "  // {}\n".format(valid_guard)
                )
                if fix:
                    fcontents = "\n".join(flines)
                    save(fpath, fcontents)
            else:
                # something else is wrong, bail out
                self.fail(
                    fpath,
                    self.endif_c_core_re if c_core_header else self.endif_re,
                    flines[-1],
                    "",
                    "",
                    False,
                )
        elif match[-1] != running_guard:
            # Is the #endif guard the same as the #ifndef and #define guards?
            fcontents = self.fail(
                fpath, self.endif_re, fcontents, match[-1], valid_guard, fix
            )
            if fix:
                save(fpath, fcontents)

        match = self.comments_then_includes_re.search(fcontents)
        assert match
        bad_includes = match.group(3)
        if bad_includes:
            print(
                "includes after initial comments but before include guards in",
                fpath,
            )
            if fix:
                fcontents = (
                    fcontents[: match.start(3)]
                    + match.group(5)
                    + match.group(3)
                    + fcontents[match.end(5) :]
                )
                save(fpath, fcontents)

        return not self.failed  # Did the check succeed? (ie, not failed)


# find our home
ROOT = os.path.abspath(os.path.join(os.path.dirname(sys.argv[0]), "../.."))
os.chdir(ROOT)

# parse command line
argp = argparse.ArgumentParser(description="include guard checker")
argp.add_argument("-f", "--fix", default=False, action="store_true")
argp.add_argument("--precommit", default=False, action="store_true")
args = argp.parse_args()

grep_filter = (
    r"grep -E '^(include|src/core|src/cpp|test/core|test/cpp|fuzztest/)/.*\.h$'"
)
if args.precommit:
    git_command = "git diff --name-only HEAD"
else:
    git_command = "git ls-tree -r --name-only -r HEAD"

FILE_LIST_COMMAND = " | ".join((git_command, grep_filter))

# scan files
ok = True
filename_list = []
try:
    filename_list = (
        subprocess.check_output(FILE_LIST_COMMAND, shell=True)
        .decode()
        .splitlines()
    )
    # Filter out non-existent files (ie, file removed or renamed)
    filename_list = (f for f in filename_list if os.path.isfile(f))
except subprocess.CalledProcessError:
    sys.exit(0)

validator = GuardValidator()

for filename in filename_list:
    # Skip check for upb generated code.
    if (
        filename.endswith(".upb.h")
        or filename.endswith(".upbdefs.h")
        or filename.endswith(".upbdefs.c")
        or filename.endswith(".upb_minitable.h")
        or filename.endswith(".upb_minitable.c")
    ):
        continue
    ok = ok and validator.check(filename, args.fix)

sys.exit(0 if ok else 1)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests behavior around the compression mechanism."""

import asyncio
import logging
import platform
import random
import unittest

import grpc
from grpc.experimental import aio

from tests_aio.unit import _common
from tests_aio.unit._test_base import AioTestBase

_GZIP_CHANNEL_ARGUMENT = ("grpc.default_compression_algorithm", 2)
_GZIP_DISABLED_CHANNEL_ARGUMENT = (
    "grpc.compression_enabled_algorithms_bitset",
    3,
)
_DEFLATE_DISABLED_CHANNEL_ARGUMENT = (
    "grpc.compression_enabled_algorithms_bitset",
    5,
)

_TEST_UNARY_UNARY = "/test/TestUnaryUnary"
_TEST_SET_COMPRESSION = "/test/TestSetCompression"
_TEST_DISABLE_COMPRESSION_UNARY = "/test/TestDisableCompressionUnary"
_TEST_DISABLE_COMPRESSION_STREAM = "/test/TestDisableCompressionStream"

_REQUEST = b"\x01" * 100
_RESPONSE = b"\x02" * 100


async def _test_unary_unary(unused_request, unused_context):
    return _RESPONSE


async def _test_set_compression(unused_request_iterator, context):
    assert _REQUEST == await context.read()
    context.set_compression(grpc.Compression.Deflate)
    await context.write(_RESPONSE)
    try:
        context.set_compression(grpc.Compression.Deflate)
    except RuntimeError:
        # NOTE(lidiz) Testing if the servicer context raises exception when
        # the set_compression method is called after initial_metadata sent.
        # After the initial_metadata sent, the server-side has no control over
        # which compression algorithm it should use.
        pass
    else:
        raise ValueError(
            "Expecting exceptions if set_compression is not effective"
        )


async def _test_disable_compression_unary(request, context):
    assert _REQUEST == request
    context.set_compression(grpc.Compression.Deflate)
    context.disable_next_message_compression()
    return _RESPONSE


async def _test_disable_compression_stream(unused_request_iterator, context):
    assert _REQUEST == await context.read()
    context.set_compression(grpc.Compression.Deflate)
    await context.write(_RESPONSE)
    context.disable_next_message_compression()
    await context.write(_RESPONSE)
    await context.write(_RESPONSE)


_ROUTING_TABLE = {
    _TEST_UNARY_UNARY: grpc.unary_unary_rpc_method_handler(_test_unary_unary),
    _TEST_SET_COMPRESSION: grpc.stream_stream_rpc_method_handler(
        _test_set_compression
    ),
    _TEST_DISABLE_COMPRESSION_UNARY: grpc.unary_unary_rpc_method_handler(
        _test_disable_compression_unary
    ),
    _TEST_DISABLE_COMPRESSION_STREAM: grpc.stream_stream_rpc_method_handler(
        _test_disable_compression_stream
    ),
}


class _GenericHandler(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        return _ROUTING_TABLE.get(handler_call_details.method)


async def _start_test_server(options=None):
    server = aio.server(options=options)
    port = server.add_insecure_port("[::]:0")
    server.add_generic_rpc_handlers((_GenericHandler(),))
    await server.start()
    return f"localhost:{port}", server


class TestCompression(AioTestBase):
    async def setUp(self):
        server_options = (_GZIP_DISABLED_CHANNEL_ARGUMENT,)
        self._address, self._server = await _start_test_server(server_options)
        self._channel = aio.insecure_channel(self._address)

    async def tearDown(self):
        await self._channel.close()
        await self._server.stop(None)

    async def test_channel_level_compression_baned_compression(self):
        # GZIP is disabled, this call should fail
        async with aio.insecure_channel(
            self._address, compression=grpc.Compression.Gzip
        ) as channel:
            multicallable = channel.unary_unary(_TEST_UNARY_UNARY)
            call = multicallable(_REQUEST)
            with self.assertRaises(aio.AioRpcError) as exception_context:
                await call
            rpc_error = exception_context.exception
            self.assertEqual(grpc.StatusCode.UNIMPLEMENTED, rpc_error.code())

    async def test_channel_level_compression_allowed_compression(self):
        # Deflate is allowed, this call should succeed
        async with aio.insecure_channel(
            self._address, compression=grpc.Compression.Deflate
        ) as channel:
            multicallable = channel.unary_unary(_TEST_UNARY_UNARY)
            call = multicallable(_REQUEST)
            self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def test_client_call_level_compression_baned_compression(self):
        multicallable = self._channel.unary_unary(_TEST_UNARY_UNARY)

        # GZIP is disabled, this call should fail
        call = multicallable(_REQUEST, compression=grpc.Compression.Gzip)
        with self.assertRaises(aio.AioRpcError) as exception_context:
            await call
        rpc_error = exception_context.exception
        self.assertEqual(grpc.StatusCode.UNIMPLEMENTED, rpc_error.code())

    async def test_client_call_level_compression_allowed_compression(self):
        multicallable = self._channel.unary_unary(_TEST_UNARY_UNARY)

        # Deflate is allowed, this call should succeed
        call = multicallable(_REQUEST, compression=grpc.Compression.Deflate)
        self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def test_server_call_level_compression(self):
        multicallable = self._channel.stream_stream(_TEST_SET_COMPRESSION)
        call = multicallable()
        await call.write(_REQUEST)
        await call.done_writing()
        self.assertEqual(_RESPONSE, await call.read())
        self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def test_server_disable_compression_unary(self):
        multicallable = self._channel.unary_unary(
            _TEST_DISABLE_COMPRESSION_UNARY
        )
        call = multicallable(_REQUEST)
        self.assertEqual(_RESPONSE, await call)
        self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def test_server_disable_compression_stream(self):
        multicallable = self._channel.stream_stream(
            _TEST_DISABLE_COMPRESSION_STREAM
        )
        call = multicallable()
        await call.write(_REQUEST)
        await call.done_writing()
        self.assertEqual(_RESPONSE, await call.read())
        self.assertEqual(_RESPONSE, await call.read())
        self.assertEqual(_RESPONSE, await call.read())
        self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def test_server_default_compression_algorithm(self):
        server = aio.server(compression=grpc.Compression.Deflate)
        port = server.add_insecure_port("[::]:0")
        server.add_generic_rpc_handlers((_GenericHandler(),))
        await server.start()

        async with aio.insecure_channel(f"localhost:{port}") as channel:
            multicallable = channel.unary_unary(_TEST_UNARY_UNARY)
            call = multicallable(_REQUEST)
            self.assertEqual(_RESPONSE, await call)
            self.assertEqual(grpc.StatusCode.OK, await call.code())

        await server.stop(None)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Patches the spawn() command for windows compilers.

Windows has an 8191 character command line limit, but some compilers
support an @command_file directive where command_file is a file
containing the full command line.
"""

from distutils import ccompiler
import os
import os.path
import shutil
import sys
import tempfile

MAX_COMMAND_LENGTH = 8191

_classic_spawn = ccompiler.CCompiler.spawn


def escape_arg(arg):
    # Escape single backslash with double backslash
    arg = arg.replace("\\", "\\\\")

    # Escape double quotes with a backslash
    arg = arg.replace('"', '\\"')

    return f'"{arg}"'


def _commandfile_spawn(self, command, **kwargs):
    if os.name == "nt":
        if any(arg.startswith("/Tc") for arg in command):
            # Remove /std:c++17 option if this is a MSVC C complation
            command = [arg for arg in command if arg != "/std:c++17"]
        elif any(arg.startswith("/Tp") for arg in command):
            # Remove /std:c11 option if this is a MSVC C++ complation
            command = [arg for arg in command if arg != "/std:c11"]

    enable_ccache = os.environ.get("GRPC_BUILD_ENABLE_CCACHE", "").lower() in [
        "true",
        "1",
    ]
    has_ccache = shutil.which("ccache") is not None
    use_ccache = enable_ccache and has_ccache and command[0].endswith("cl.exe")

    if use_ccache:
        # Workaround for ccache 4.8 not handling certain flags properly:

        new_command = []
        for arg in command:
            # /Tp, /Tc:
            #    ccache performs a preprocessor run to generate cacheable
            #    file, and /Tp /Tc flags seems to be repositioned such that
            #    the msvc backend doesn't recognize it.
            #    e.g. /SomeFlag /Tpfile.cc -> /Tp /SomeFlag file.cc
            if arg.startswith("/Tp") and len(arg) > 3:
                # Replace with the global variant. This is fine for
                # setuptools build because it only builds one file at
                # a time.
                new_command.extend(["/TP", arg[3:]])
            elif arg.startswith("/Tc") and len(arg) > 3:
                new_command.extend(["/TC", arg[3:]])
            #   /Zc:preprocessor:
            #      This is not supported by ccache but needed for protobuf 33.5
            #      See also https://github.com/grpc/grpc/issues/41951
            elif arg == "/Zc:preprocessor":
                new_command.extend(["--ccache-skip", "/Zc:preprocessor"])
            else:
                new_command.append(arg)
        command = new_command

    command_length = sum([len(arg) for arg in command])
    if os.name == "nt" and command_length > MAX_COMMAND_LENGTH:
        # Even if this command doesn't support the @command_file, it will
        # fail as is so we try blindly
        print("Command line length exceeded, using command file")
        print(" ".join(command))
        temporary_directory = tempfile.mkdtemp()
        command_filename = os.path.abspath(
            os.path.join(temporary_directory, "command")
        )
        with open(command_filename, "w") as command_file:
            escaped_args = map(escape_arg, command[1:])
            # add each arg on a separate line to avoid hitting the
            # "line in command file contains 131071 or more characters" error
            # (can happen for extra long link commands)
            command_file.write(" \n".join(escaped_args))

        if use_ccache:
            modified_command = (
                ["ccache"] + command[:1] + ["@{}".format(command_filename)]
            )
        else:
            modified_command = command[:1] + ["@{}".format(command_filename)]

        try:
            _classic_spawn(self, modified_command, **kwargs)
        finally:
            shutil.rmtree(temporary_directory)
    else:
        if use_ccache:
            _classic_spawn(self, ["ccache"] + command, **kwargs)
        else:
            _classic_spawn(self, command, **kwargs)


def monkeypatch_spawn():
    """Monkeypatching is dumb, but it's either that or we become maintainers of
    something much, much bigger."""
    ccompiler.CCompiler.spawn = _commandfile_spawn



# Copyright 2020 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

def _contextvars_supported():
    """Determines if the contextvars module is supported.

    We use a 'try it and see if it works approach' here rather than predicting
    based on interpreter version in order to support older interpreters that
    may have a backported module based on, e.g. `threading.local`.

    Returns:
      A bool indicating whether `contextvars` are supported in the current
      environment.
    """
    try:
        import contextvars
        return True
    except ImportError:
        return False


def _run_with_context(target):
    """Runs a callable with contextvars propagated.

    If contextvars are supported, the calling thread's context will be copied
    and propagated. If they are not supported, this function is equivalent
    to the identity function.

    Args:
      target: A callable object to wrap.
    Returns:
      A callable object with the same signature as `target` but with
        contextvars propagated.
    """


if _contextvars_supported():
    import contextvars
    def _run_with_context(target):
        ctx = contextvars.copy_context()
        def _run(*args):
            ctx.run(target, *args)
        return _run
else:
    def _run_with_context(target):
        def _run(*args):
            target(*args)
        return _run



# Copyright 2024 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
from typing import AnyStr, Callable, Dict, Iterable, List, Optional, Union

from google.protobuf import struct_pb2
from grpc_observability._observability import OptionalLabelType
from grpc_observability._open_telemetry_plugin import OpenTelemetryLabelInjector
from grpc_observability._open_telemetry_plugin import OpenTelemetryPlugin
from grpc_observability._open_telemetry_plugin import OpenTelemetryPluginOption
from opentelemetry.metrics import MeterProvider
from opentelemetry.resourcedetector.gcp_resource_detector import (
    GoogleCloudResourceDetector,
)
from opentelemetry.sdk.resources import Resource
from opentelemetry.semconv.resource import ResourceAttributes

TRAFFIC_DIRECTOR_AUTHORITY = "traffic-director-global.xds.googleapis.com"
UNKNOWN_VALUE = "unknown"
TYPE_GCE = "gcp_compute_engine"
TYPE_GKE = "gcp_kubernetes_engine"
MESH_ID_PREFIX = "mesh:"

METADATA_EXCHANGE_KEY_FIXED_MAP = {
    "type": "csm.remote_workload_type",
    "canonical_service": "csm.remote_workload_canonical_service",
}

METADATA_EXCHANGE_KEY_GKE_MAP = {
    "workload_name": "csm.remote_workload_name",
    "namespace_name": "csm.remote_workload_namespace_name",
    "cluster_name": "csm.remote_workload_cluster_name",
    "location": "csm.remote_workload_location",
    "project_id": "csm.remote_workload_project_id",
}

METADATA_EXCHANGE_KEY_GCE_MAP = {
    "workload_name": "csm.remote_workload_name",
    "location": "csm.remote_workload_location",
    "project_id": "csm.remote_workload_project_id",
}


class CSMOpenTelemetryLabelInjector(OpenTelemetryLabelInjector):
    """
    An implementation of OpenTelemetryLabelInjector for CSM.

    This injector will fetch labels from GCP resource detector and
    environment, it's also responsible for serialize and deserialize
    metadata exchange labels.
    """

    _exchange_labels: Dict[str, AnyStr]
    _additional_exchange_labels: Dict[str, str]

    def __init__(self):
        fields = {}
        self._exchange_labels = {}
        self._additional_exchange_labels = {}

        # Labels from environment
        canonical_service_value = os.getenv(
            "CSM_CANONICAL_SERVICE_NAME", UNKNOWN_VALUE
        )
        workload_name_value = os.getenv("CSM_WORKLOAD_NAME", UNKNOWN_VALUE)
        mesh_id = os.getenv("CSM_MESH_ID", UNKNOWN_VALUE)

        gcp_resource = GoogleCloudResourceDetector().detect()
        resource_type_value = get_resource_type(gcp_resource)
        namespace_value = get_str_value_from_resource(
            ResourceAttributes.K8S_NAMESPACE_NAME, gcp_resource
        )
        cluster_name_value = get_str_value_from_resource(
            ResourceAttributes.K8S_CLUSTER_NAME, gcp_resource
        )
        # ResourceAttributes.CLOUD_AVAILABILITY_ZONE are called
        # "zones" on Google Cloud.
        location_value = get_str_value_from_resource("cloud.zone", gcp_resource)
        if location_value == UNKNOWN_VALUE:
            location_value = get_str_value_from_resource(
                ResourceAttributes.CLOUD_REGION, gcp_resource
            )
        project_id_value = get_str_value_from_resource(
            ResourceAttributes.CLOUD_ACCOUNT_ID, gcp_resource
        )

        fields["type"] = struct_pb2.Value(string_value=resource_type_value)
        fields["canonical_service"] = struct_pb2.Value(
            string_value=canonical_service_value
        )
        if resource_type_value == TYPE_GKE:
            fields["workload_name"] = struct_pb2.Value(
                string_value=workload_name_value
            )
            fields["namespace_name"] = struct_pb2.Value(
                string_value=namespace_value
            )
            fields["cluster_name"] = struct_pb2.Value(
                string_value=cluster_name_value
            )
            fields["location"] = struct_pb2.Value(string_value=location_value)
            fields["project_id"] = struct_pb2.Value(
                string_value=project_id_value
            )
        elif resource_type_value == TYPE_GCE:
            fields["workload_name"] = struct_pb2.Value(
                string_value=workload_name_value
            )
            fields["location"] = struct_pb2.Value(string_value=location_value)
            fields["project_id"] = struct_pb2.Value(
                string_value=project_id_value
            )

        serialized_struct = struct_pb2.Struct(fields=fields)
        serialized_str = serialized_struct.SerializeToString()

        self._exchange_labels = {"XEnvoyPeerMetadata": serialized_str}
        self._additional_exchange_labels["csm.workload_canonical_service"] = (
            canonical_service_value
        )
        self._additional_exchange_labels["csm.mesh_id"] = mesh_id

    def get_labels_for_exchange(self) -> Dict[str, AnyStr]:
        return self._exchange_labels

    def get_additional_labels(
        self, include_exchange_labels: bool
    ) -> Dict[str, str]:
        if include_exchange_labels:
            return self._additional_exchange_labels
        return {}

    @staticmethod
    def deserialize_labels(labels: Dict[str, AnyStr]) -> Dict[str, AnyStr]:
        deserialized_labels = {}
        for key, value in labels.items():
            if key == "XEnvoyPeerMetadata":
                pb_struct = struct_pb2.Struct()
                pb_struct.ParseFromString(value)

                remote_type = get_value_from_struct("type", pb_struct)

                for (
                    local_key,
                    remote_key,
                ) in METADATA_EXCHANGE_KEY_FIXED_MAP.items():
                    deserialized_labels[remote_key] = get_value_from_struct(
                        local_key, pb_struct
                    )
                if remote_type == TYPE_GKE:
                    for (
                        local_key,
                        remote_key,
                    ) in METADATA_EXCHANGE_KEY_GKE_MAP.items():
                        deserialized_labels[remote_key] = get_value_from_struct(
                            local_key, pb_struct
                        )
                elif remote_type == TYPE_GCE:
                    for (
                        local_key,
                        remote_key,
                    ) in METADATA_EXCHANGE_KEY_GCE_MAP.items():
                        deserialized_labels[remote_key] = get_value_from_struct(
                            local_key, pb_struct
                        )
            # If CSM label injector is enabled on server side but client didn't send
            # XEnvoyPeerMetadata, we'll record remote label as unknown.
            else:
                for remote_key in METADATA_EXCHANGE_KEY_FIXED_MAP.values():
                    deserialized_labels[remote_key] = UNKNOWN_VALUE
                deserialized_labels[key] = value

        return deserialized_labels


class CsmOpenTelemetryPluginOption(OpenTelemetryPluginOption):
    """
    An implementation of OpenTelemetryPlugin for CSM.
    """

    _label_injector: CSMOpenTelemetryLabelInjector

    def __init__(self):
        self._label_injector = CSMOpenTelemetryLabelInjector()

    @staticmethod
    def is_active_on_client_channel(target: str) -> bool:
        """Determines whether this plugin option is active on a channel based on target.

        Args:
          target: Required. The target for the RPC.

        Returns:
          True if this plugin option is active on the channel, false otherwise.
        """
        # CSM channels should have an "xds" scheme
        if not target.startswith("xds:"):
            return False
        # If scheme is correct, the authority should be TD if exist
        authority_pattern = r"^xds:\/\/([^/]+)"
        match = re.search(authority_pattern, target)
        if match:
            return TRAFFIC_DIRECTOR_AUTHORITY in match.group(1)
        # Return True if the authority doesn't exist
        return True

    @staticmethod
    def is_active_on_server(
        xds: bool,  # pylint: disable=unused-argument #noqa: ARG004
    ) -> bool:
        """Determines whether this plugin option is active on a given server.

        Since servers don't need to be xds enabled to work as part of a service
        mesh, we're returning True and enable this PluginOption for all servers.

        Note: This always returns true because server can be part of the mesh even
        if it's not xds-enabled. And we want CSM labels for those servers too.

        Args:
          xds: Required. if this server is build for xds.

        Returns:
          True if this plugin option is active on the server, false otherwise.
        """
        return True

    def get_label_injector(self) -> OpenTelemetryLabelInjector:
        return self._label_injector


# pylint: disable=no-self-use
class CsmOpenTelemetryPlugin(OpenTelemetryPlugin):
    """Describes a Plugin for CSM OpenTelemetry observability.

    This is class is part of an EXPERIMENTAL API.
    """

    plugin_options: Iterable[OpenTelemetryPluginOption]
    meter_provider: Optional[MeterProvider]
    generic_method_attribute_filter: Callable[[str], bool]

    def __init__(
        self,
        *,
        plugin_options: Optional[Iterable[OpenTelemetryPluginOption]] = None,
        meter_provider: Optional[MeterProvider] = None,
        generic_method_attribute_filter: Optional[Callable[[str], bool]] = None,
    ):
        plugin_options = plugin_options or []
        new_options = list(plugin_options) + [CsmOpenTelemetryPluginOption()]
        super().__init__(
            plugin_options=new_options,
            meter_provider=meter_provider,
            generic_method_attribute_filter=generic_method_attribute_filter,
        )

    def _get_enabled_optional_labels(self) -> List[OptionalLabelType]:
        return [OptionalLabelType.XDS_SERVICE_LABELS]


def get_value_from_struct(key: str, struct: struct_pb2.Struct) -> str:
    value = struct.fields.get(key)
    if not value:
        return UNKNOWN_VALUE
    return value.string_value


def get_str_value_from_resource(
    attribute: Union[ResourceAttributes, str], resource: Resource
) -> str:
    value = resource.attributes.get(attribute, UNKNOWN_VALUE)
    return str(value)


# pylint: disable=line-too-long
def get_resource_type(gcp_resource: Resource) -> str:
    # Convert resource type from GoogleCloudResourceDetector to the value we used for
    # metadata exchange.
    # Reference: https://github.com/GoogleCloudPlatform/opentelemetry-operations-python/blob/cc61f23a5ff2f16f4aa2c38d07e55153828849cc/opentelemetry-resourcedetector-gcp/src/opentelemetry/resourcedetector/gcp_resource_detector/__init__.py#L96
    gcp_resource_type = get_str_value_from_resource(
        "gcp.resource_type", gcp_resource
    )
    if gcp_resource_type == "gke_container":
        return TYPE_GKE
    if gcp_resource_type == "gce_instance":
        return TYPE_GCE
    return gcp_resource_type



# Copyright 2019 the gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Proxies a TCP connection between a single client-server pair.

This proxy is not suitable for production, but should work well for cases in
which a test needs to spy on the bytes put on the wire between a server and
a client.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import datetime
import select
import socket
import threading

from tests.unit.framework.common import get_socket

_TCP_PROXY_BUFFER_SIZE = 1024
_TCP_PROXY_TIMEOUT = datetime.timedelta(milliseconds=500)


def _init_proxy_socket(gateway_address, gateway_port):
    proxy_socket = socket.create_connection((gateway_address, gateway_port))
    return proxy_socket


class TcpProxy:
    """Proxies a TCP connection between one client and one server."""

    def __init__(self, bind_address, gateway_address, gateway_port):
        self._bind_address = bind_address
        self._gateway_address = gateway_address
        self._gateway_port = gateway_port

        self._byte_count_lock = threading.RLock()
        self._sent_byte_count = 0
        self._received_byte_count = 0

        self._stop_event = threading.Event()

        self._port = None
        self._listen_socket = None
        self._proxy_socket = None

        # The following three attributes are owned by the serving thread.
        self._northbound_data = b""
        self._southbound_data = b""
        self._client_sockets = []

        self._thread = threading.Thread(target=self._run_proxy)

    def start(self):
        _, self._port, self._listen_socket = get_socket(
            bind_address=self._bind_address
        )
        self._proxy_socket = _init_proxy_socket(
            self._gateway_address, self._gateway_port
        )
        self._thread.start()

    def get_port(self):
        return self._port

    def _handle_reads(self, sockets_to_read):
        for socket_to_read in sockets_to_read:
            if socket_to_read is self._listen_socket:
                client_socket, client_address = socket_to_read.accept()
                self._client_sockets.append(client_socket)
            elif socket_to_read is self._proxy_socket:
                data = socket_to_read.recv(_TCP_PROXY_BUFFER_SIZE)
                with self._byte_count_lock:
                    self._received_byte_count += len(data)
                self._northbound_data += data
            elif socket_to_read in self._client_sockets:
                data = socket_to_read.recv(_TCP_PROXY_BUFFER_SIZE)
                if data:
                    with self._byte_count_lock:
                        self._sent_byte_count += len(data)
                    self._southbound_data += data
                else:
                    self._client_sockets.remove(socket_to_read)
            else:
                raise RuntimeError("Unidentified socket appeared in read set.")

    def _handle_writes(self, sockets_to_write):
        for socket_to_write in sockets_to_write:
            if socket_to_write is self._proxy_socket:
                if self._southbound_data:
                    self._proxy_socket.sendall(self._southbound_data)
                    self._southbound_data = b""
            elif socket_to_write in self._client_sockets:
                if self._northbound_data:
                    socket_to_write.sendall(self._northbound_data)
                    self._northbound_data = b""

    def _run_proxy(self):
        while not self._stop_event.is_set():
            expected_reads = (self._listen_socket, self._proxy_socket) + tuple(
                self._client_sockets
            )
            expected_writes = expected_reads
            sockets_to_read, sockets_to_write, _ = select.select(
                expected_reads,
                expected_writes,
                (),
                _TCP_PROXY_TIMEOUT.total_seconds(),
            )
            self._handle_reads(sockets_to_read)
            self._handle_writes(sockets_to_write)
        for client_socket in self._client_sockets:
            client_socket.close()

    def stop(self):
        self._stop_event.set()
        self._thread.join()
        self._listen_socket.close()
        self._proxy_socket.close()

    def get_byte_count(self):
        with self._byte_count_lock:
            return self._sent_byte_count, self._received_byte_count

    def reset_byte_count(self):
        with self._byte_count_lock:
            self._byte_count = 0
            self._received_byte_count = 0

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()



# Copyright 2018 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


def channelz_get_top_channels(start_channel_id):
    cdef char *c_returned_str = grpc_channelz_get_top_channels(
        start_channel_id,
    )
    if c_returned_str == NULL:
        raise ValueError('Failed to get top channels, please ensure your' \
                         ' start_channel_id==%s is valid' % start_channel_id)
    return c_returned_str
    
def channelz_get_servers(start_server_id):
    cdef char *c_returned_str = grpc_channelz_get_servers(start_server_id)
    if c_returned_str == NULL:
        raise ValueError('Failed to get servers, please ensure your' \
                         ' start_server_id==%s is valid' % start_server_id)
    return c_returned_str
    
def channelz_get_server(server_id):
    cdef char *c_returned_str = grpc_channelz_get_server(server_id)
    if c_returned_str == NULL:
        raise ValueError('Failed to get the server, please ensure your' \
                         ' server_id==%s is valid' % server_id)
    return c_returned_str
    
def channelz_get_server_sockets(server_id, start_socket_id, max_results):
    cdef char *c_returned_str = grpc_channelz_get_server_sockets(
        server_id,
        start_socket_id,
        max_results,
    )
    if c_returned_str == NULL:
        raise ValueError('Failed to get server sockets, please ensure your' \
                         ' server_id==%s and start_socket_id==%s and' \
                         ' max_results==%s is valid' %
                         (server_id, start_socket_id, max_results))
    return c_returned_str
    
def channelz_get_channel(channel_id):
    cdef char *c_returned_str = grpc_channelz_get_channel(channel_id)
    if c_returned_str == NULL:
        raise ValueError('Failed to get the channel, please ensure your' \
                         ' channel_id==%s is valid' % (channel_id))
    return c_returned_str
    
def channelz_get_subchannel(subchannel_id):
    cdef char *c_returned_str = grpc_channelz_get_subchannel(subchannel_id)
    if c_returned_str == NULL:
        raise ValueError('Failed to get the subchannel, please ensure your' \
                         ' subchannel_id==%s is valid' % (subchannel_id))
    return c_returned_str
    
def channelz_get_socket(socket_id):
    cdef char *c_returned_str = grpc_channelz_get_socket(socket_id)
    if c_returned_str == NULL:
        raise ValueError('Failed to get the socket, please ensure your' \
                         ' socket_id==%s is valid' % (socket_id))
    return c_returned_str



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from envoy.service.network_ext_proc.v3 import network_external_processor_pb2 as envoy_dot_service_dot_network__ext__proc_dot_v3_dot_network__external__processor__pb2

GRPC_GENERATED_VERSION = '1.74.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in envoy/service/network_ext_proc/v3/network_external_processor_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class NetworkExternalProcessorStub(object):
    """[#protodoc-title: Network External Processing Service]

    The Network External Processing filter allows an external service to dynamically
    interact with and modify L4 network traffic passing through Envoy. Unlike the
    HTTP External Processing filter, this service operates at the TCP/UDP level,
    providing access to raw network data.

    The filter communicates with an external gRPC service that can:

    1. Inspect network traffic in both directions (client->server and server->client)
    2. Modify the payload data
    3. Control connection lifecycle (continue, close gracefully, or reset)

    Use cases include:

    1. Custom protocol inspection and modification
    2. Advanced traffic manipulation
    3. Security scanning and filtering
    4. Dynamic connection management

    The service uses a bidirectional gRPC stream, maintaining state throughout
    the connection lifetime while allowing asynchronous processing.

    NetworkExternalProcessor service defines the contract between Envoy and
    external processing servers for L4 network traffic.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Process = channel.stream_stream(
                '/envoy.service.network_ext_proc.v3.NetworkExternalProcessor/Process',
                request_serializer=envoy_dot_service_dot_network__ext__proc_dot_v3_dot_network__external__processor__pb2.ProcessingRequest.SerializeToString,
                response_deserializer=envoy_dot_service_dot_network__ext__proc_dot_v3_dot_network__external__processor__pb2.ProcessingResponse.FromString,
                _registered_method=True)


class NetworkExternalProcessorServicer(object):
    """[#protodoc-title: Network External Processing Service]

    The Network External Processing filter allows an external service to dynamically
    interact with and modify L4 network traffic passing through Envoy. Unlike the
    HTTP External Processing filter, this service operates at the TCP/UDP level,
    providing access to raw network data.

    The filter communicates with an external gRPC service that can:

    1. Inspect network traffic in both directions (client->server and server->client)
    2. Modify the payload data
    3. Control connection lifecycle (continue, close gracefully, or reset)

    Use cases include:

    1. Custom protocol inspection and modification
    2. Advanced traffic manipulation
    3. Security scanning and filtering
    4. Dynamic connection management

    The service uses a bidirectional gRPC stream, maintaining state throughout
    the connection lifetime while allowing asynchronous processing.

    NetworkExternalProcessor service defines the contract between Envoy and
    external processing servers for L4 network traffic.
    """

    def Process(self, request_iterator, context):
        """Process establishes a bidirectional stream between Envoy and the external
        processing server. Envoy sends ProcessingRequests containing network data
        and the server responds with ProcessingResponses containing processing
        decisions and potentially modified data.

        The server should handle processing timeout properly to avoid blocking
        network traffic for extended periods. Any uncaught exceptions will
        be treated as errors and will terminate the stream.

        Implementation note: The server should process requests in the order
        they are received to maintain proper sequencing of network traffic.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_NetworkExternalProcessorServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Process': grpc.stream_stream_rpc_method_handler(
                    servicer.Process,
                    request_deserializer=envoy_dot_service_dot_network__ext__proc_dot_v3_dot_network__external__processor__pb2.ProcessingRequest.FromString,
                    response_serializer=envoy_dot_service_dot_network__ext__proc_dot_v3_dot_network__external__processor__pb2.ProcessingResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'envoy.service.network_ext_proc.v3.NetworkExternalProcessor', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('envoy.service.network_ext_proc.v3.NetworkExternalProcessor', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class NetworkExternalProcessor(object):
    """[#protodoc-title: Network External Processing Service]

    The Network External Processing filter allows an external service to dynamically
    interact with and modify L4 network traffic passing through Envoy. Unlike the
    HTTP External Processing filter, this service operates at the TCP/UDP level,
    providing access to raw network data.

    The filter communicates with an external gRPC service that can:

    1. Inspect network traffic in both directions (client->server and server->client)
    2. Modify the payload data
    3. Control connection lifecycle (continue, close gracefully, or reset)

    Use cases include:

    1. Custom protocol inspection and modification
    2. Advanced traffic manipulation
    3. Security scanning and filtering
    4. Dynamic connection management

    The service uses a bidirectional gRPC stream, maintaining state throughout
    the connection lifetime while allowing asynchronous processing.

    NetworkExternalProcessor service defines the contract between Envoy and
    external processing servers for L4 network traffic.
    """

    @staticmethod
    def Process(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(
            request_iterator,
            target,
            '/envoy.service.network_ext_proc.v3.NetworkExternalProcessor/Process',
            envoy_dot_service_dot_network__ext__proc_dot_v3_dot_network__external__processor__pb2.ProcessingRequest.SerializeToString,
            envoy_dot_service_dot_network__ext__proc_dot_v3_dot_network__external__processor__pb2.ProcessingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Provides setuptools command classes for the GRPC Python setup process."""

import os
import shutil

import setuptools

ROOT_DIR = os.path.abspath(os.path.dirname(os.path.abspath(__file__)))
GRPC_ROOT_ABS_PATH = os.path.join(ROOT_DIR, "../../..")
ROOT_REL_DIR = os.path.relpath(ROOT_DIR, start=GRPC_ROOT_ABS_PATH)
HEALTH_PROTO = "src/proto/grpc/health/v1/health.proto"
LICENSE = "./LICENSE"


class Preprocess(setuptools.Command):
    """Command to copy proto modules from grpc/src/proto and LICENSE from
    the root directory"""

    description = ""
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        # TODO: Can skip copy proto part.
        if os.path.isfile(HEALTH_PROTO):
            shutil.copyfile(
                HEALTH_PROTO,
                os.path.join(ROOT_REL_DIR, "grpc_health/v1/health.proto"),
            )
        if os.path.isfile(LICENSE):
            shutil.copyfile(LICENSE, os.path.join(ROOT_REL_DIR, "LICENSE"))


class BuildPackageProtos(setuptools.Command):
    """Command to generate project *_pb2.py modules from proto files."""

    description = "build grpc protobuf modules"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from grpc_tools import command

        # find and build all protos in the current package
        command.build_package_protos(ROOT_REL_DIR, strict_mode=True)



# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys


def get_bool_env(env_var_name: str, default: bool = False) -> bool:
    val = os.getenv(env_var_name, "").lower()
    if val in {"1", "t", "true", "y", "yes"}:
        return True
    if val in {"0", "f", "false", "n", "no"}:
        return False
    return default


def kokoro_build_initiator() -> str:
    return os.getenv("KOKORO_BUILD_INITIATOR", "")


def kokoro_build_started_by_ci() -> bool:
    # Kokoro has additional accounts, f.e. kokoro-dedicated.
    return kokoro_build_initiator().startswith("kokoro")


def should_upload_results_on_ci() -> bool:
    # Only upload results when initiated by the CI.
    #
    # Right now we differentiate by checking the KOKORO_BUILD_INITIATOR env var,
    # but it may be better to identify the GitHub repo owner and skip the upload
    # not "grpc" org, but a fork. We could use KOKORO_GITHUB_COMMIT_URL,
    # but how reliable is it for all types of jobs? Alternatively, we could
    # extract it from `git remote get-url`, but this needs to be tested too.
    #
    # Note: this prints to stderr to preserve the current behavior
    # of bazel_report_helper.py. Should probably switch to a normal logger.

    # TODO(sergiitk): should we move UPLOAD_TEST_RESULTS parsing here too?
    # But we need to be sure it's propagated correctly.
    if kokoro_build_started_by_ci():
        return True

    build_initiator = kokoro_build_initiator()

    # Allow to override this behavior.
    if get_bool_env("UPLOAD_TEST_RESULTS_FORCE"):
        print(
            "[TEST UPLOAD] Force-enabling test results upload"
            f" for non-CI user {build_initiator!r}",
            file=sys.stderr,
        )
        return True

    print(
        "[TEST UPLOAD] Disabling test results upload for non-CI user"
        f" {build_initiator!r}. Enable with UPLOAD_TEST_RESULTS_FORCE=true",
        file=sys.stderr,
    )
    return False
